#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "../lib/Eic.h"


int
main (int argc, char *argv[])
{
  Eic_Prop       *eic;
  Eic_Video_Prop *vp;
  Eic_Audio_Prop *ap;
  GList          *list;
  char           *str;
  char            container[128];
  double          err1;
  double          err2;
  int             nb_authors;
  int             i;

  if (argc <  2) {
    printf ("Usage: %s filename\n\n", argv[0]);
    return -1;
  }

  eic = eic_new (argv[1]);
  if (!eic) {
    printf ("%s: can't allocate global structure. Exiting...\n\n", argv[0]);
    return -1;
  }
  if (!eic_fill (eic)) {
    eic_free (eic);
    return -1;
  }

  printf ("\n");
  printf ("Filename....................: %s\n", eic->filename);
  printf ("Size........................: ");
  str = get_size_string (eic->size);
  printf ("%s\n", str);
  free (str);
  printf ("Container...................: ");
  if (eic->demuxer_info) {
    str = strchr (eic->demuxer_info->name, ' ');
    i = 127 < (str - eic->demuxer_info->name) ? 127 : str - eic->demuxer_info->name;
    strncpy (container, eic->demuxer_info->name, i);
    container[i] = '\0';
    printf ("%s\n", container);
    printf ("Demuxer info\n");
    printf ("  |\n");
    printf ("  +-Name....................: %s\n", eic->demuxer_info->name);
    printf ("  +-Description.............: %s\n", eic->demuxer_info->description);
    printf ("  +-Class...................: %s\n", eic->demuxer_info->class);
    for (nb_authors = 0 ; eic->demuxer_info->authors[nb_authors] ; nb_authors++) {}
    if (nb_authors == 1)
      printf ("  +-Author..................: ");
    else
      printf ("  +-Authors.................: ");
    for (i = 0 ; i < nb_authors ; i++)
      {
        if (i != 0)
          printf ("                    ");
        printf ("%s\n", eic->demuxer_info->authors[i]);
      }
  }
  else {
    printf (" no informations\n");
  }

  printf ("\n");
  printf ("Streams\n");
  list = g_list_first (eic->video_prop);
  for (i = 0; list ; list = g_list_next (list), i++) {
    int len;
    int i_authors;

    vp = (Eic_Video_Prop *)list->data;
    len = g_list_length (list) + g_list_length (eic->video_prop);
    for (nb_authors = 0 ; vp->decoder_info->authors[nb_authors] ; nb_authors++) {}
    printf (" |\n");
    printf (" +-Video stream #%d\n", i);
    if (i < len-1) printf (" |");
    else printf ("  ");
    printf ("  +-Mimetype..............: %s\n", vp->mimetype);
    if (i < len-1) printf (" |");
    else printf ("  ");
    if (vp->prop) {
      switch (vp->type) {
      case EIC_VIDEO_RAW_YUV: {
        Eic_Video_Yuv_Prop *vyp;

        vyp = (Eic_Video_Yuv_Prop *)vp->prop;
        printf ("  +-FourCC................: %s\n", vyp->fourcc);
        if (i < len-1) printf (" |");
        else printf ("  ");
        break;
      }
      case EIC_VIDEO_RAW_RGB: {
        Eic_Video_Rgb_Prop *vrp;

        vrp = (Eic_Video_Rgb_Prop *)vp->prop;
        printf ("  +-bpp...................: %d\n", vrp->bpp);
        if (i < len-1) printf (" |");
        else printf ("  ");
        printf ("  +-Depth.................: %d\n", vrp->depth);
        if (i < len-1) printf (" |");
        else printf ("  ");
        printf ("  +-Endianess.............: %d\n", vrp->endianess);
        if (i < len-1) printf (" |");
        else printf ("  ");
        printf ("  +-Red mask..............: %d\n", vrp->red_mask);
        if (i < len-1) printf (" |");
        else printf ("  ");
        printf ("  +-Green mask............: %d\n", vrp->green_mask);
        if (i < len-1) printf (" |");
        else printf ("  ");
        printf ("  +-Blue mask.............: %d\n", vrp->blue_mask);
        if (i < len-1) printf (" |");
        else printf ("  ");
        break;
      }
      }
    }
    printf ("  +-Decoder info\n");
    if (i < len-1) printf (" |  |  |\n");
    else printf ("    |  |\n");

    if (i < len-1) printf (" |  |  +-");
    else printf ("    |  +-");
    printf ("Name...............: %s\n", vp->decoder_info->name);
    if (i < len-1) printf (" |  |  +-");
    else printf ("    |  +-");
    printf ("Description........: %s\n", vp->decoder_info->description);
    if (i < len-1) printf (" |  |  +-");
    else printf ("    |  +-");
    printf ("Class..............: %s\n", vp->decoder_info->class);
    if (i < len-1) printf (" |  |  +-");
    else printf ("    |  +-");
    if (nb_authors == 1)
      printf ("Author.............: ");
    else
      printf ("Authors............: ");
    for (i_authors = 0 ; i_authors < nb_authors ; i_authors++)
      {
        if (i_authors != 0)
          {
            if (i_authors < len-1) printf (" |  |                    ");
            else printf ("    |                    ");
          }
        printf ("%s\n", vp->decoder_info->authors[i_authors]);
      }

    if (i < len-1) printf (" |  |\n");
    else printf ("    |\n");
    if (i < len-1) printf (" |");
    else printf ("  ");
    printf ("  +-Duration..............: ");
    printf ("%u:%02u:%02u.%03u\n", EIC_TIME_ARGS (vp->length_time));
    if (i < len-1) printf (" |");
    else printf ("  ");
    printf ("  +-Framerate.............: %d/%d\n", vp->framerate_num, vp->framerate_den);
    if (i < len-1) printf (" |");
    else printf ("  ");
    printf ("  +-Frame count...........: %lld\n", vp->length_frames);
    if (i < len-1) printf (" |");
    else printf ("  ");
    printf ("  +-Size..................: ");
    printf ("%dx%d (%.3f)",
            vp->width, vp->height,
            (double)vp->width/(double)vp->height);

    if (vp->width*9 == vp->height*16)
      printf (" [16:9]");
    else
      {
        if (vp->width*3 == vp->height*4)
          printf (" [4:3]");
        else
          {
            err1 = ((double)(2*abs(vp->width*3 - vp->height*4)) /
                    (double)((vp->width*3 + vp->height*4)));
            err2 = ((double)(2*abs(vp->width*9 - vp->height*16)) /
                    (double)((vp->width*9 + vp->height*16)));
            if (err1 < err2)
              {
                if (err1*100.0 < 3)
                  printf (" {4:3}");
              }
            else
              {
                if (err2*100.0 < 3)
                  printf (" {16:9}");
              }
          }
      }
    printf ("\n");
    if (i < len-1) printf (" |");
    else printf ("  ");
    printf ("  +-Pixel aspect ratio....: %d/%d\n", vp->par_num, vp->par_den);
    if (i < len-1) printf (" |");
    else printf ("  ");
    printf ("  +-Display aspect ratio..: %d/%d (%.3f)\n", vp->dar_num ,vp->dar_den, (double)vp->dar_num / (double)vp->dar_den);
  }

  list = g_list_first (eic->audio_prop);
  for (i = 0; list ; list = g_list_next (list), i++) {
    int len;
    int i_authors;

    ap = (Eic_Audio_Prop *)list->data;
    len = g_list_length (list);
    for (nb_authors = 0 ; ap->decoder_info->authors[nb_authors] ; nb_authors++) {}
    printf (" |\n");
    printf (" +-Audio stream #%d\n", i);
    if (i < len-1) printf (" |");
    else printf ("  ");
    printf ("  +-Mimetype..............: %s\n", ap->mimetype);

    if (i < len-1) printf (" |");
    else printf ("  ");
    printf ("  +-Decoder info\n");
    if (i < len-1) printf (" |  |  |\n");
    else printf ("    |  |\n");

    if (i < len-1) printf (" |  |  +-");
    else printf ("    |  +-");
    printf ("Name...............: %s\n", ap->decoder_info->name);
    if (i < len-1) printf (" |  |  +-");
    else printf ("    |  +-");
    printf ("Description........: %s\n", ap->decoder_info->description);
    if (i < len-1) printf (" |  |  +-");
    else printf ("    |  +-");
    printf ("Class..............: %s\n", ap->decoder_info->class);
    if (i < len-1) printf (" |  |  +-");
    else printf ("    |  +-");
    if (nb_authors == 1)
      printf ("Author.............: ");
    else
      printf ("Authors............: ");
    for (i_authors = 0 ; i_authors < nb_authors ; i_authors++)
      {
        if (i_authors != 0)
          {
            if (i < len-1) printf (" |  |                    ");
            else printf ("    |                    ");
          }
        printf ("%s\n", ap->decoder_info->authors[i_authors]);
      }

    if (i < len-1) printf (" |  |\n");
    else printf ("    |\n");
    if (i < len-1) printf (" |");
    else printf ("  ");
    printf ("  +-Duration..............: ");
    printf ("%u:%02u:%02u.%03u\n", EIC_TIME_ARGS (ap->length_time));
    if (i < len-1) printf (" |");
    else printf ("  ");
    printf ("  +-Sample rate...........: %d\n", ap->samplerate);
    if (i < len-1) printf (" |");
    else printf ("  ");
    printf ("  +-Sample count..........: %lld\n", ap->length_samples);
    if (i < len-1) printf (" |");
    else printf ("  ");
    printf ("  +-Channels..............: %d\n", ap->channels);

    if (ap->prop) {
      switch (ap->type) {
      case EIC_AUDIO_RAW_INT: {
        Eic_Audio_Int_Prop *aip;

        aip = (Eic_Audio_Int_Prop *)ap->prop;
        if (i < len-1) printf (" |");
        else printf ("  ");
        printf ("  +-Endianess.............: %d\n", aip->endianess);
        if (i < len-1) printf (" |");
        else printf ("  ");
        printf ("  +-Width.................: %d\n", aip->width);
        if (i < len-1) printf (" |");
        else printf ("  ");
        printf ("  +-Depth.................: %d\n", aip->depth);
        if (i < len-1) printf (" |");
        else printf ("  ");
        printf ("  +-Signed................: ");
        if (aip->signed_samples)
          printf ("yes\n");
        else
          printf ("no\n");
        break;
      }
      case EIC_AUDIO_RAW_FLOAT: {
        Eic_Audio_Float_Prop *afp;

        afp = (Eic_Audio_Float_Prop *)ap->prop;
        if (i < len-1) printf (" |");
        else printf ("  ");
        printf ("  +-Endianess.............: %d\n", afp->endianess);
        if (i < len-1) printf (" |");
        else printf ("  ");
        printf ("  +-Width.................: %d\n", afp->width);
        if (i < len-1) printf (" |");
        else printf ("  ");
        printf ("  +-Buffer frames.........: %d\n", afp->buffer_frames);
        break;
      }
      }
    }
  }

  eic_free (eic);

  return 0;
}
