#include <stdlib.h>
#include <string.h>

#include <Etk.h>

#include "../lib/Eic.h"


typedef struct Eic_Etk_Factory_Widgets_ Eic_Etk_Factory_Widgets;
typedef struct Eic_Etk_Audio_Widgets_ Eic_Etk_Audio_Widgets;
typedef struct Eic_Etk_Video_Widgets_ Eic_Etk_Video_Widgets;
typedef struct Eic_Etk_Widgets_ Eic_Etk_Widgets;

struct Eic_Etk_Factory_Widgets_
{
  Etk_Widget *name;
  Etk_Widget *description;
  Etk_Widget *class;
};

struct Eic_Etk_Audio_Widgets_
{
  Etk_Widget              *list;
  Eic_Etk_Factory_Widgets *decoder;
  Etk_Widget              *samplerate;
  Etk_Widget              *channels;
  Etk_Widget              *length_time;
  Etk_Widget              *length_samples;
};

struct Eic_Etk_Video_Widgets_
{
  Etk_Widget              *list;
  Eic_Etk_Factory_Widgets *decoder;
  Etk_Widget              *resolution;
  Etk_Widget              *framerate;
  Etk_Widget              *par;
  Etk_Widget              *dar;
  Etk_Widget              *length_time;
  Etk_Widget              *length_frames;
};

struct Eic_Etk_Widgets_
{
  Eic_Prop                *eic;

  Etk_Widget              *filename;
  Etk_Widget              *filesize;
  Etk_Widget              *container;

  Eic_Etk_Video_Widgets   *video;
  Eic_Etk_Audio_Widgets   *audio;
};

/* Callbacks */
static void eic_etk_open_cb (Etk_Object *object, void *data);
static void eic_etk_fd_response_cb (Etk_Widget *widget, int res, void *data);
static void eic_etk_quit_cb (Etk_Object *object, void *data);
static void eic_etk_video_fill_cb (Etk_Object *object, void *data);
static void eic_etk_audio_fill_cb (Etk_Object *object, void *data);

void
eic_etk_factory_info_attach (Etk_Widget       *parent,
                             Eic_Factory_Info *factory)
{
  char *str;
  char  buffer[4096];
  int   nb_authors;
  int   index;

  if (!factory)
    return;

  for (nb_authors = 0 ; factory->authors[nb_authors] ; nb_authors++) {}
  if (nb_authors == 1)
    str = "Author........: ";
  else
    str = "Authors.......: ";

  g_snprintf (buffer, 4096,
             "Name..........: %s\nDescription...: %s\nClass.........: %s\n%s%s",
             factory->name,
             factory->description,
             factory->class,
             str,
             factory->authors[0]);
  for (index = 1 ; index < nb_authors ; index++) {
    g_snprintf (buffer, 4096, "%s%s%s\n",
               buffer,
               "\n                    ",
               factory->authors[index]);
  }
  etk_tooltips_tip_set (parent, buffer);
}


Eic_Etk_Video_Widgets *
eic_etk_video_init (Eic_Etk_Widgets *eic_etk)
{
  Eic_Etk_Video_Widgets *video;

  video = (Eic_Etk_Video_Widgets *)malloc (sizeof (Eic_Etk_Video_Widgets));
  if (!video)
    return NULL;

  video->list = etk_combobox_new_default ();
  etk_signal_connect("active_item_changed", ETK_OBJECT(video->list), ETK_CALLBACK(eic_etk_video_fill_cb), eic_etk);
  video->decoder = (Eic_Etk_Factory_Widgets *)malloc (sizeof (Eic_Etk_Factory_Widgets));
  video->decoder->name = etk_entry_new ();
  video->resolution = etk_entry_new ();
  video->framerate = etk_entry_new ();
  video->par = etk_entry_new ();
  video->dar = etk_entry_new ();
  video->length_time = etk_entry_new ();
  video->length_frames = etk_entry_new ();

  return video;
}

void
eic_etk_video_clear (Eic_Etk_Video_Widgets *video, Eic_Prop *eic)
{
  etk_combobox_clear (ETK_COMBOBOX (video->list));
  etk_entry_text_set (ETK_ENTRY (video->decoder->name), "");
  etk_entry_text_set (ETK_ENTRY (video->resolution), "");
  etk_entry_text_set (ETK_ENTRY (video->framerate), "");
  etk_entry_text_set (ETK_ENTRY (video->par), "");
  etk_entry_text_set (ETK_ENTRY (video->dar), "");
  etk_entry_text_set (ETK_ENTRY (video->length_time), "");
  etk_entry_text_set (ETK_ENTRY (video->length_frames), "");
}

Eic_Etk_Audio_Widgets *
eic_etk_audio_init (Eic_Etk_Widgets *eic_etk)
{
  Eic_Etk_Audio_Widgets *audio;

  audio = (Eic_Etk_Audio_Widgets *)malloc (sizeof (Eic_Etk_Audio_Widgets));
  if (!audio)
    return NULL;

  audio->list = etk_combobox_new_default ();
  etk_signal_connect("active_item_changed", ETK_OBJECT(audio->list), ETK_CALLBACK(eic_etk_audio_fill_cb), eic_etk);
  audio->decoder = (Eic_Etk_Factory_Widgets *)malloc (sizeof (Eic_Etk_Factory_Widgets));
  audio->decoder->name = etk_entry_new ();
  audio->samplerate = etk_entry_new ();
  audio->channels = etk_entry_new ();
  audio->length_time = etk_entry_new ();
  audio->length_samples = etk_entry_new ();

  return audio;
}

void
eic_etk_audio_clear (Eic_Etk_Audio_Widgets *audio, Eic_Prop *eic)
{
  etk_combobox_clear (ETK_COMBOBOX (audio->list));
  etk_entry_text_set (ETK_ENTRY (audio->decoder->name), "");
  etk_entry_text_set (ETK_ENTRY (audio->samplerate), "");
  etk_entry_text_set (ETK_ENTRY (audio->channels), "");
  etk_entry_text_set (ETK_ENTRY (audio->length_time), "");
  etk_entry_text_set (ETK_ENTRY (audio->length_samples), "");
}

Eic_Etk_Widgets *
eic_etk_init ()
{
  Eic_Etk_Widgets *eic_etk;

  eic_etk = (Eic_Etk_Widgets *)malloc (sizeof (Eic_Etk_Widgets));
  if (!eic_etk)
    return NULL;

  eic_etk->eic = NULL;
  eic_etk->filename = etk_entry_new ();
  eic_etk->filesize = etk_entry_new ();
  eic_etk->container = etk_entry_new ();
/*   eic_etk->demuxer; */
  eic_etk->video = eic_etk_video_init (eic_etk);
  if (!eic_etk->video) {
    free (eic_etk);
    return NULL;
  }
  eic_etk->audio = eic_etk_audio_init (eic_etk);
  if (!eic_etk->audio) {
    free (eic_etk->video);
    free (eic_etk);
    return NULL;
  }

  etk_tooltips_init ();
  etk_tooltips_enable ();

  return eic_etk;
}

void
eic_etk_free (Eic_Etk_Widgets *eic_etk)
{
  if (!eic_etk)
    return;

  if (eic_etk->video) {
    if (eic_etk->video->decoder)
      free (eic_etk->video->decoder);
    free (eic_etk->video);
  }

  if (eic_etk->audio) {
    if (eic_etk->audio->decoder)
      free (eic_etk->audio->decoder);
    free (eic_etk->audio);
  }

  free (eic_etk);
}

static void
eic_etk_video_fill_cb (Etk_Object *object, void *data)
{
  char                   buffer[4096];
  Eic_Etk_Widgets       *eic_etk;
  Eic_Prop              *eic;
  Eic_Video_Prop        *eic_video;
  Eic_Etk_Video_Widgets *eic_etk_video;
  GList                 *elt;
  char                  *str;
  double                 err1;
  double                 err2;
  double                 err;

  eic_etk = (Eic_Etk_Widgets *)data;
  eic = eic_etk->eic;
  if (!eic)
    return;

  eic_etk_video = eic_etk->video;
  elt = g_list_nth (eic->video_prop,
                    etk_combobox_active_item_get_index (ETK_COMBOBOX (object)));
  if (!elt)
    return;

  eic_video = (Eic_Video_Prop *)elt->data;
  if (!eic_video)
    return;

  etk_entry_text_set (ETK_ENTRY (eic_etk_video->decoder->name), eic_video->decoder_info->name);
  eic_etk_factory_info_attach (eic_etk_video->decoder->name, eic_video->decoder_info);
/*   etk_entry_text_set (ETK_ENTRY (eic_etk_video->mimetype), eic_video->mimetype); */
  snprintf (buffer, 128, "%dx%d", eic_video->width, eic_video->height);
  etk_entry_text_set (ETK_ENTRY (eic_etk_video->resolution), buffer);
  snprintf (buffer, 128, "%d/%d", eic_video->framerate_num, eic_video->framerate_den);
  etk_entry_text_set (ETK_ENTRY (eic_etk_video->framerate), buffer);
  snprintf (buffer, 128, "%dx%d", eic_video->par_num, eic_video->par_den);
  etk_entry_text_set (ETK_ENTRY (eic_etk_video->par), buffer);
  snprintf (buffer, 128, "%u:%02u:%02u.%03u", EIC_TIME_ARGS (eic_video->length_time));
  etk_entry_text_set (ETK_ENTRY (eic_etk_video->length_time), buffer);
  snprintf (buffer, 128, "%lld", eic_video->length_frames);
  etk_entry_text_set (ETK_ENTRY (eic_etk_video->length_frames), buffer);

  if ((eic_video->dar_num > 0) && (eic_video->dar_den)) {
    err = -1.0;
    str = "";
    if (eic_video->dar_num*9 == eic_video->dar_den*16)
      str = " [16:9]";
    else {
      if (eic_video->dar_num*3 == eic_video->dar_den*4)
        str = " [4:3]";
      else {
        err1 = ((double)(2 * abs (eic_video->dar_num*3 - eic_video->dar_den * 4)) /
                (double)((eic_video->dar_num * 3 + eic_video->dar_den * 4)));
        err2 = ((double)(2 * abs (eic_video->dar_num * 9 - eic_video->dar_den * 16)) /
                (double)((eic_video->dar_num * 9 + eic_video->dar_den * 16)));
        if (err1 < err2) {
          if (err1 * 100.0 < 3) {
            str = " {4:3}";
            err = 100.0 * err1;
          }
        }
        else {
          if (err2 * 100.0 < 3) {
            str = " {16:9}";
            err = 100.0 * err2;;
          }
        }
      }
    }

    snprintf (buffer, 4096, "%dx%d (%.3f:1) %s",
              eic_video->dar_num, eic_video->dar_den,
              (double)eic_video->dar_num / (double)eic_video->dar_den,
              str);
    str = g_strdup (buffer);
    etk_entry_text_set (ETK_ENTRY (eic_etk_video->dar), str);

    if (err < 0.0)
      snprintf (buffer, 4096,
                "%s\n[*:*] means exact ratio\n{*:*} means approximative ratio",
                str);
    else
      snprintf (buffer, 4096,
                "%s\n[*:*] means exact ratio\n{*:*} means approximative ratio (%.1f %%)",
                str, err);
    g_free (str);
   etk_tooltips_tip_set(eic_etk_video->dar,  buffer);
  }
}

void
eic_etk_audio_fill_cb (Etk_Object *object, void *data)
{
  char                   buffer[128];
  Eic_Etk_Widgets       *eic_etk;
  Eic_Prop              *eic;
  Eic_Audio_Prop        *eic_audio;
  Eic_Etk_Audio_Widgets *eic_etk_audio;
  GList                 *elt;

  eic_etk = (Eic_Etk_Widgets *)data;
  eic = eic_etk->eic;
  if (!eic)
    return;

  eic_etk_audio = eic_etk->audio;
  elt = g_list_nth (eic->audio_prop,
                    etk_combobox_active_item_get_index (ETK_COMBOBOX (object)));
  if (!elt)
    return;

  eic_audio = (Eic_Audio_Prop *)elt->data;
  if (!eic_audio)
    return;

/*   etk_entry_text_set (ETK_ENTRY (eic_etk_audio->mimetype), eic_audio->mimetype); */
  etk_entry_text_set (ETK_ENTRY (eic_etk_audio->decoder->name), eic_audio->decoder_info->name);
  eic_etk_factory_info_attach (eic_etk_audio->decoder->name, eic_audio->decoder_info);
  snprintf (buffer, 128, "%d", eic_audio->samplerate);
  etk_entry_text_set (ETK_ENTRY (eic_etk_audio->samplerate), buffer);
  snprintf (buffer, 128, "%d", eic_audio->channels);
  etk_entry_text_set (ETK_ENTRY (eic_etk_audio->channels), buffer);
  snprintf (buffer, 128, "%u:%02u:%02u.%03u", EIC_TIME_ARGS (eic_audio->length_time));
  etk_entry_text_set (ETK_ENTRY (eic_etk_audio->length_time), buffer);
  snprintf (buffer, 128, "%lld", eic_audio->length_samples);
  etk_entry_text_set (ETK_ENTRY (eic_etk_audio->length_samples), buffer);
}

void
eic_etk_fill (Eic_Etk_Widgets *eic_etk)
{
  Eic_Prop *eic;
  GList    *list;
  char     *str;

  eic = eic_etk->eic;
  if (!eic)
    return;

  etk_entry_text_set (ETK_ENTRY (eic_etk->filename), eic->filename);

  str = get_size_string (eic->size);
  etk_entry_text_set (ETK_ENTRY (eic_etk->filesize), str);
  free (str);

  if (eic->demuxer_info) {
    char container[128];
    int  i;

    str = strchr (eic->demuxer_info->name, ' ');
    i = 127 < (str - eic->demuxer_info->name) ? 127 : str - eic->demuxer_info->name;
    strncpy (container, eic->demuxer_info->name, i);
    container[i] = '\0';
    etk_entry_text_set (ETK_ENTRY (eic_etk->container), container);
    eic_etk_factory_info_attach (eic_etk->container, eic->demuxer_info);
  }
  else {
    etk_entry_text_set (ETK_ENTRY (eic_etk->container), "no informations");
  }

  /* video */
  eic_etk_video_clear (eic_etk->video, eic);
  list = g_list_first (eic->video_prop);
  for (; list ; list = g_list_next (list)) {
    Eic_Video_Prop *vp;
    gchar           buffer[1024];

    vp = (Eic_Video_Prop *)list->data;
    g_snprintf (buffer, 1024, "%s (%d/%d)",
                vp->mimetype,
                g_list_position (g_list_first (list), list) + 1,
                g_list_length (g_list_first (list)));
    etk_combobox_item_append (ETK_COMBOBOX (eic_etk->video->list), buffer);
  }

  /* audio */
  eic_etk_audio_clear (eic_etk->audio, eic);
  list = g_list_first (eic->audio_prop);
  for (; list ; list = g_list_next (list)) {
    Eic_Audio_Prop *ap;
    gchar           buffer[1024];

    ap = (Eic_Audio_Prop *)list->data;
    g_snprintf (buffer, 1024, "%s (%d/%d)",
                ap->mimetype,
                g_list_position (g_list_first (list), list) + 1,
                g_list_length (g_list_first (list)));
    etk_combobox_item_append (ETK_COMBOBOX (eic_etk->audio->list), buffer);
  }
/*   eic_etk_video_fill (eic_etk, 0); */
/*   eic_etk_audio_fill (eic_etk, 0); */
}

Etk_Widget *
eic_etk_video_frame (Eic_Etk_Widgets *eic_etk)
{
  Etk_Widget *frame;
  Etk_Widget *table;
  Etk_Widget *label;
  Eic_Etk_Video_Widgets *video;

  if (!eic_etk)
    return NULL;

  video = eic_etk->video;

  frame = etk_frame_new ("Video properties");

  table = etk_table_new (2, 8, ETK_TRUE);
  etk_container_add (ETK_CONTAINER (frame), table);
  etk_widget_show (table);

  label = etk_label_new ("Stream:");
  etk_table_attach_defaults (ETK_TABLE (table), label, 0, 0, 0, 0);
  etk_widget_show (label);

  etk_table_attach_defaults (ETK_TABLE (table), video->list, 1, 1, 0, 0);
  etk_widget_show (video->list);

  label = etk_label_new ("Decoder:");
  etk_table_attach_defaults (ETK_TABLE (table), label, 0, 0, 1, 1);
  etk_widget_show (label);

  etk_table_attach_defaults (ETK_TABLE (table), video->decoder->name, 1, 1, 1, 1);
  etk_widget_show (video->decoder->name);

  label = etk_label_new ("Resolution:");
  etk_table_attach_defaults (ETK_TABLE (table), label, 0, 0, 2, 2);
  etk_widget_show (label);

  etk_table_attach_defaults (ETK_TABLE (table), video->resolution, 1, 1, 2, 2);
  etk_widget_show (video->resolution);

  label = etk_label_new ("Frame rate:");
  etk_table_attach_defaults (ETK_TABLE (table), label, 0, 0, 3, 3);
  etk_widget_show (label);

  etk_table_attach_defaults (ETK_TABLE (table), video->framerate, 1, 1, 3, 3);
  etk_widget_show (video->framerate);

  label = etk_label_new ("Pixel aspect ratio:");
  etk_table_attach_defaults (ETK_TABLE (table), label, 0, 0, 4, 4);
  etk_widget_show (label);

  etk_table_attach_defaults (ETK_TABLE (table), video->par, 1, 1, 4, 4);
  etk_widget_show (video->par);

  label = etk_label_new ("Display aspect ratio:");
  etk_table_attach_defaults (ETK_TABLE (table), label, 0, 0, 5, 5);
  etk_widget_show (label);

  etk_table_attach_defaults (ETK_TABLE (table), video->dar, 1, 1, 5, 5);
  etk_widget_show (video->dar);

  label = etk_label_new ("Duration:");
  etk_table_attach_defaults (ETK_TABLE (table), label, 0, 0, 6, 6);
  etk_widget_show (label);

  etk_table_attach_defaults (ETK_TABLE (table), video->length_time, 1, 1, 6, 6);
  etk_widget_show (video->length_time);

  label = etk_label_new ("Frame count:");
  etk_table_attach_defaults (ETK_TABLE (table), label, 0, 0, 7, 7);
  etk_widget_show (label);

  etk_table_attach_defaults (ETK_TABLE (table), video->length_frames, 1, 1, 7, 7);
  etk_widget_show (video->length_frames);

  return frame;
}

Etk_Widget *
eic_etk_audio_frame (Eic_Etk_Widgets *eic_etk)
{
  Etk_Widget *frame;
  Etk_Widget *table;
  Etk_Widget *label;
  Eic_Etk_Audio_Widgets *audio;

  if (!eic_etk)
    return NULL;

  audio = eic_etk->audio;

  frame = etk_frame_new ("Audio properties");

  table = etk_table_new (2, 6, ETK_TRUE);
  etk_container_add (ETK_CONTAINER (frame), table);
  etk_widget_show (table);

  label = etk_label_new ("Stream:");
  etk_table_attach_defaults (ETK_TABLE (table), label, 0, 0, 0, 0);
  etk_widget_show (label);

  etk_table_attach_defaults (ETK_TABLE (table), audio->list, 1, 1, 0, 0);
  etk_widget_show (audio->list);

  label = etk_label_new ("Decoder:");
  etk_table_attach_defaults (ETK_TABLE (table), label, 0, 0, 1, 1);
  etk_widget_show (label);

  etk_table_attach_defaults (ETK_TABLE (table), audio->decoder->name, 1, 1, 1, 1);
  etk_widget_show (audio->decoder->name);

  label = etk_label_new ("Sample rate:");
  etk_table_attach_defaults (ETK_TABLE (table), label, 0, 0, 2, 2);
  etk_widget_show (label);

  etk_table_attach_defaults (ETK_TABLE (table), audio->samplerate, 1, 1, 2, 2);
  etk_widget_show (audio->samplerate);

  label = etk_label_new ("Channels:");
  etk_table_attach_defaults (ETK_TABLE (table), label, 0, 0, 3, 3);
  etk_widget_show (label);

  etk_table_attach_defaults (ETK_TABLE (table), audio->channels, 1, 1, 3, 3);
  etk_widget_show (audio->channels);

  label = etk_label_new ("Duration:");
  etk_table_attach_defaults (ETK_TABLE (table), label, 0, 0, 4, 4);
  etk_widget_show (label);

  etk_table_attach_defaults (ETK_TABLE (table), audio->length_time, 1, 1, 4, 4);
  etk_widget_show (audio->length_time);

  label = etk_label_new ("Sample count:");
  etk_table_attach_defaults (ETK_TABLE (table), label, 0, 0, 5, 5);
  etk_widget_show (label);

  etk_table_attach_defaults (ETK_TABLE (table), audio->length_samples, 1, 1, 5, 5);
  etk_widget_show (audio->length_samples);

  return frame;
}

Etk_Widget *
eic_etk_general_frame (Eic_Etk_Widgets *eic_etk)
{
  Etk_Widget *frame;
  Etk_Widget *table;
  Etk_Widget *label;

  frame = etk_frame_new ("General");

  table = etk_table_new (2, 3, ETK_TRUE);
  etk_container_add (ETK_CONTAINER (frame), table);
  etk_widget_show (table);

  label = etk_label_new ("Filename:");
  etk_table_attach_defaults (ETK_TABLE (table), label, 0, 0, 0, 0);
  etk_widget_show (label);

  etk_table_attach_defaults (ETK_TABLE (table), eic_etk->filename, 1, 1, 0, 0);
  etk_widget_show (eic_etk->filename);

  label = etk_label_new ("File size:");
  etk_table_attach_defaults (ETK_TABLE (table), label, 0, 0, 1, 1);
  etk_widget_show (label);

  etk_table_attach_defaults (ETK_TABLE (table), eic_etk->filesize, 1, 1, 1, 1);
  etk_widget_show (eic_etk->filesize);

  label = etk_label_new ("Container:");
  etk_table_attach_defaults (ETK_TABLE (table), label, 0, 0, 2, 2);
  etk_widget_show (label);

  etk_table_attach_defaults (ETK_TABLE (table), eic_etk->container, 1, 1, 2, 2);
  etk_widget_show (eic_etk->container);

  return frame;
}

void
eic_etk_main_window (Eic_Etk_Widgets *eic_etk)
{
  Etk_Widget *win;
  Etk_Widget *vbox;
  Etk_Widget *menu_bar;
  Etk_Widget *menu;
  Etk_Widget *menu_item;
  Etk_Widget *frame;

  win = etk_window_new ();
  etk_window_title_set(ETK_WINDOW(win), "Eic - Enlightened Information Codec");
  etk_signal_connect("delete_event", ETK_OBJECT(win), ETK_CALLBACK(eic_etk_quit_cb), eic_etk);

  vbox = etk_vbox_new (ETK_FALSE, 6);
  etk_container_add (ETK_CONTAINER (win), vbox);
  etk_widget_show (vbox);

  menu_bar = etk_menu_bar_new();
  etk_box_pack_start(ETK_BOX(vbox), menu_bar, ETK_FALSE, ETK_FALSE, 0);
  etk_widget_show (menu_bar);

  /* File menu */
  menu_item = etk_menu_item_new_with_label("File");
  etk_menu_shell_append(ETK_MENU_SHELL(menu_bar), ETK_MENU_ITEM(menu_item));
  menu = etk_menu_new();
  etk_menu_item_submenu_set(ETK_MENU_ITEM(menu_item), ETK_MENU(menu));
  menu_item = etk_menu_item_new_with_label("Open");
  etk_menu_shell_append(ETK_MENU_SHELL(menu), ETK_MENU_ITEM(menu_item));
  etk_signal_connect("activated", ETK_OBJECT(menu_item), ETK_CALLBACK(eic_etk_open_cb), eic_etk);
  menu_item = etk_menu_item_separator_new();
  etk_menu_shell_append(ETK_MENU_SHELL(menu), ETK_MENU_ITEM(menu_item));
  menu_item = etk_menu_item_new_with_label("Quit");
  etk_menu_shell_append(ETK_MENU_SHELL(menu), ETK_MENU_ITEM(menu_item));
  etk_signal_connect("activated", ETK_OBJECT(menu_item), ETK_CALLBACK(eic_etk_quit_cb), eic_etk);

  /* Help menu */
  menu_item = etk_menu_item_new_with_label("Help");
  etk_menu_shell_append(ETK_MENU_SHELL(menu_bar), ETK_MENU_ITEM(menu_item));
  menu = etk_menu_new();
  etk_menu_item_submenu_set(ETK_MENU_ITEM(menu_item), ETK_MENU(menu));
  menu_item = etk_menu_item_new_with_label("About");
  etk_menu_shell_append(ETK_MENU_SHELL(menu), ETK_MENU_ITEM(menu_item));

  frame = eic_etk_general_frame (eic_etk);
  etk_box_pack_start (ETK_BOX (vbox), frame, ETK_TRUE, ETK_TRUE, 0);
  etk_widget_show (frame);

  frame = eic_etk_video_frame (eic_etk);
  etk_box_pack_start (ETK_BOX (vbox), frame, ETK_TRUE, ETK_TRUE, 0);
  etk_widget_show (frame);

  frame = eic_etk_audio_frame (eic_etk);
  etk_box_pack_start (ETK_BOX (vbox), frame, ETK_TRUE, ETK_TRUE, 0);
  etk_widget_show (frame);

  etk_widget_show (win);
}

int
main (int argc, char *argv[])
{
  Eic_Etk_Widgets *eic_etk;

  etk_init (argc, argv);

  eic_etk = eic_etk_init ();
  if (!eic_etk) {
    etk_main_quit ();
    return -1;
  }

  eic_etk_main_window (eic_etk);

  if (argc > 1) {
    eic_etk->eic = eic_new (argv[1]);
    eic_fill (eic_etk->eic);
    eic_etk_fill (eic_etk);
  }

  etk_main ();

  return 0;
}

static void
eic_etk_open_cb(Etk_Object *object, void *data)
{
  static Etk_Widget *dialog;
  Etk_Widget *fc;

  dialog = etk_dialog_new ();
  etk_window_title_set (ETK_WINDOW (dialog), "Choose a file");
  etk_dialog_button_add_from_stock (ETK_DIALOG (dialog), ETK_STOCK_DOCUMENT_OPEN, 1);
  etk_dialog_button_add_from_stock (ETK_DIALOG (dialog), ETK_STOCK_DIALOG_CLOSE, 2);
  etk_signal_connect("delete_event", ETK_OBJECT (dialog),
                     ETK_CALLBACK (etk_window_hide_on_delete), NULL);
  etk_signal_connect("response", ETK_OBJECT (dialog),
                     ETK_CALLBACK (eic_etk_fd_response_cb), data);

  fc = etk_filechooser_widget_new ();
  etk_object_data_set (ETK_OBJECT (dialog), "filechooser", fc);
  etk_dialog_pack_in_main_area (ETK_DIALOG (dialog), fc,
                                ETK_TRUE, ETK_TRUE, 0, ETK_FALSE);
  etk_widget_show (fc);

  etk_widget_show (dialog);
}

static void
eic_etk_fd_response_cb (Etk_Widget *widget, int res, void *data)
{
  switch (res) {
  case 1: {
    Etk_Filechooser_Widget *fc;
    Eic_Etk_Widgets        *eic_etk;
    const char             *path;
    const char             *filename;
    char *file;
    int l1;
    int l2;

    etk_widget_hide (widget);

    fc = ETK_FILECHOOSER_WIDGET (etk_object_data_get (ETK_OBJECT (widget), "filechooser"));
    path = etk_filechooser_widget_current_folder_get (fc);
    filename = etk_filechooser_widget_selected_file_get (fc);
    if (!path || !filename)
      return;

    l1 = strlen (path);
    l2 = strlen (filename);
    file = (char *)malloc (sizeof (char) * (l1 + l2 + 2));
    memcpy (file, path, l1);
    file[l1] = '/';
    memcpy (file + l1 + 1, filename, l2);
    file[l1 + l2 + 1] = '\0';

    eic_etk = (Eic_Etk_Widgets *)data;
    eic_etk->eic = eic_new (file);
    eic_fill (eic_etk->eic);
    eic_etk_fill (eic_etk);
    free (file);

    break;
  }
  default: break;
    etk_widget_hide (widget);
  }
}

static void
eic_etk_quit_cb(Etk_Object *object, void *data)
{
  Eic_Etk_Widgets *eic_etk;

  eic_etk = (Eic_Etk_Widgets *)data;
  eic_free (eic_etk->eic);
  eic_etk_free (eic_etk);
  etk_main_quit();
}
