#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <Ewl.h>

#include "../lib/Eic.h"


typedef struct Eic_Ewl_Factory_Widgets_ Eic_Ewl_Factory_Widgets;
typedef struct Eic_Ewl_Audio_Widgets_ Eic_Ewl_Audio_Widgets;
typedef struct Eic_Ewl_Video_Widgets_ Eic_Ewl_Video_Widgets;
typedef struct Eic_Ewl_Widgets_ Eic_Ewl_Widgets;

struct Eic_Ewl_Factory_Widgets_
{
  Ewl_Widget *name;
  Ewl_Widget *description;
  Ewl_Widget *class;
};

struct Eic_Ewl_Audio_Widgets_
{
  Ewl_Widget              *list;
  Eic_Ewl_Factory_Widgets *decoder;
  Ewl_Widget              *samplerate;
  Ewl_Widget              *channels;
  Ewl_Widget              *length_time;
  Ewl_Widget              *length_samples;
};

struct Eic_Ewl_Video_Widgets_
{
  Ewl_Widget              *list;
  Eic_Ewl_Factory_Widgets *decoder;
  Ewl_Widget              *resolution;
  Ewl_Widget              *framerate;
  Ewl_Widget              *par;
  Ewl_Widget              *dar;
  Ewl_Widget              *length_time;
  Ewl_Widget              *length_frames;
};

struct Eic_Ewl_Widgets_
{
  Eic_Prop                *eic;

  Ewl_Widget              *filename;
  Ewl_Widget              *filesize;
  Ewl_Widget              *container;

  Eic_Ewl_Factory_Widgets *demuxer;

  Eic_Ewl_Video_Widgets   *video;
  Eic_Ewl_Audio_Widgets   *audio;
};


/* Callbacks */
static void eic_ewl_open_cb (Ewl_Widget *widget, void *ev_data, void *data);
static void eic_ewl_fd_response_cb (Ewl_Widget *widget, void *ev_data, void *data);
static void eic_ewl_quit_cb (Ewl_Widget *widget, void *ev_data, void *data);

static void eic_ewl_video_fill_cb (Ewl_Widget *widget, void *ev_data, void *data);
static void eic_ewl_audio_fill_cb (Ewl_Widget *widget, void *ev_data, void *data);

/* Combo model functions */
static void *video_container_fetch (void *data, unsigned int row, unsigned int column);
static int video_container_count_get (void *data);

static void *audio_container_fetch (void *data, unsigned int row, unsigned int column);
static int audio_container_count_get (void *data);

/* Combo view functions */
static Ewl_Widget *video_container_name ();
static void video_container_name_set (Ewl_Widget *w, void *data);
static Ewl_Widget *video_container_header_fetch (void *data, int
column);

static Ewl_Widget *audio_container_name ();
static void audio_container_name_set (Ewl_Widget *w, void *data);
static Ewl_Widget *audio_container_header_fetch (void *data, int column);


Eic_Ewl_Video_Widgets *
eic_ewl_video_init (Eic_Ewl_Widgets *eic_ewl)
{
  Eic_Ewl_Video_Widgets *video;

  video = (Eic_Ewl_Video_Widgets *)malloc (sizeof (Eic_Ewl_Video_Widgets));
  if (!video)
    return NULL;

  video->list = ewl_combo_new ();
  ewl_callback_append (video->list,
                       EWL_CALLBACK_VALUE_CHANGED,
                       eic_ewl_video_fill_cb, eic_ewl);
  video->decoder = (Eic_Ewl_Factory_Widgets *)malloc (sizeof (Eic_Ewl_Factory_Widgets));
  video->decoder->name = ewl_entry_new ();
  video->resolution = ewl_entry_new ();
  video->framerate = ewl_entry_new ();
  video->par = ewl_entry_new ();
  video->dar = ewl_entry_new ();
  video->length_time = ewl_entry_new ();
  video->length_frames = ewl_entry_new ();

  return video;
}

void
eic_ewl_video_clear (Eic_Ewl_Video_Widgets *video, Eic_Prop *eic)
{
/*   int i; */

/*   for (i = 0; i < g_list_length (eic->video_prop); i++) */
/*     gtk_combo_box_remove_text (GTK_COMBO_BOX (video->list), i); */
  ewl_text_text_set (EWL_TEXT (video->decoder), "");
  ewl_text_text_set (EWL_TEXT (video->resolution), "");
  ewl_text_text_set (EWL_TEXT (video->framerate), "");
  ewl_text_text_set (EWL_TEXT (video->par), "");
  ewl_text_text_set (EWL_TEXT (video->dar), "");
  ewl_text_text_set (EWL_TEXT (video->length_time), "");
  ewl_text_text_set (EWL_TEXT (video->length_frames), "");
}

Eic_Ewl_Audio_Widgets *
eic_ewl_audio_init (Eic_Ewl_Widgets *eic_ewl)
{
  Eic_Ewl_Audio_Widgets *audio;

  audio = (Eic_Ewl_Audio_Widgets *)malloc (sizeof (Eic_Ewl_Audio_Widgets));
  if (!audio)
    return NULL;

  audio->list = ewl_combo_new ();
  ewl_callback_append (audio->list,
                       EWL_CALLBACK_VALUE_CHANGED,
                       eic_ewl_audio_fill_cb, eic_ewl);
  audio->decoder = (Eic_Ewl_Factory_Widgets *)malloc (sizeof (Eic_Ewl_Factory_Widgets));
  audio->decoder->name = ewl_entry_new ();
  audio->samplerate = ewl_entry_new ();
  audio->channels = ewl_entry_new ();
  audio->length_time = ewl_entry_new ();
  audio->length_samples = ewl_entry_new ();

  return audio;
}

void
eic_ewl_audio_clear (Eic_Ewl_Audio_Widgets *audio, Eic_Prop *eic)
{
/*   int i; */

/*   for (i = 0; i < g_list_length (eic->audio_prop); i++) */
/*     ewl_combo_box_remove_text (EWL_COMBO_BOX (audio->list), i); */
  ewl_text_text_set (EWL_TEXT (audio->decoder->name), "");
  ewl_text_text_set (EWL_TEXT (audio->samplerate), "");
  ewl_text_text_set (EWL_TEXT (audio->channels), "");
  ewl_text_text_set (EWL_TEXT (audio->length_time), "");
  ewl_text_text_set (EWL_TEXT (audio->length_samples), "");
}

Eic_Ewl_Widgets *
eic_ewl_init ()
{
  Eic_Ewl_Widgets *eic_ewl;

  eic_ewl = (Eic_Ewl_Widgets *)malloc (sizeof (Eic_Ewl_Widgets));
  if (!eic_ewl)
    return NULL;

  eic_ewl->eic = NULL;
  eic_ewl->filename = ewl_entry_new ();
  eic_ewl->filesize = ewl_entry_new ();
  eic_ewl->container = ewl_entry_new ();
/*   eic_ewl->demuxer; */
  eic_ewl->video = eic_ewl_video_init (eic_ewl);
  if (!eic_ewl->video) {
    free (eic_ewl);
    return NULL;
  }
  eic_ewl->audio = eic_ewl_audio_init (eic_ewl);
  if (!eic_ewl->audio) {
    free (eic_ewl->video);
    free (eic_ewl);
    return NULL;
  }

  return eic_ewl;
}

void
eic_ewl_free (Eic_Ewl_Widgets *eic_ewl)
{
  if (!eic_ewl)
    return;

  if (eic_ewl->video) {
    if (eic_ewl->video->decoder)
      free (eic_ewl->video->decoder);
    free (eic_ewl->video);
  }

  if (eic_ewl->audio) {
    if (eic_ewl->audio->decoder)
      free (eic_ewl->audio->decoder);
    free (eic_ewl->audio);
  }

  free (eic_ewl);
}

static void
eic_ewl_video_fill_cb (Ewl_Widget *widget, void *ev_data, void *data)
{
  char                   buffer[4096];
  Eic_Ewl_Widgets       *eic_ewl;
  Eic_Prop              *eic;
  Eic_Video_Prop        *eic_video;
  Eic_Ewl_Video_Widgets *eic_ewl_video;
  GList                 *elt;
  char                  *str;
  double                 err1;
  double                 err2;
  double                 err;

  eic_ewl = (Eic_Ewl_Widgets *)data;
  eic = eic_ewl->eic;
  if (!eic)
    return;

  eic_ewl_video = eic_ewl->video;
/*   elt = g_list_nth (eic->video_prop, video_index); */
  if (!elt)
    return;

  eic_video = (Eic_Video_Prop *)elt->data;
  if (!eic_video)
    return;

  ewl_text_text_set (EWL_TEXT (eic_ewl_video->decoder), eic_video->decoder_info->name);
  snprintf (buffer, 128, "%dx%d", eic_video->width, eic_video->height);
  ewl_text_text_set (EWL_TEXT (eic_ewl_video->resolution), buffer);
  snprintf (buffer, 128, "%d/%d", eic_video->framerate_num, eic_video->framerate_den);
  ewl_text_text_set (EWL_TEXT (eic_ewl_video->framerate), buffer);
  snprintf (buffer, 128, "%dx%d", eic_video->par_num, eic_video->par_den);
  ewl_text_text_set (EWL_TEXT (eic_ewl_video->par), buffer);
  snprintf (buffer, 128, "%u:%02u:%02u.%03u", EIC_TIME_ARGS (eic_video->length_time));
  ewl_text_text_set (EWL_TEXT (eic_ewl_video->length_time), buffer);
  snprintf (buffer, 128, "%lld", eic_video->length_frames);
  ewl_text_text_set (EWL_TEXT (eic_ewl_video->length_frames), buffer);

  if ((eic_video->dar_num > 0) && (eic_video->dar_den)) {
    err = -1.0;
    str = "";
    if (eic_video->dar_num*9 == eic_video->dar_den*16)
      str = " [16:9]";
    else {
      if (eic_video->dar_num*3 == eic_video->dar_den*4)
        str = " [4:3]";
      else {
        err1 = ((double)(2 * abs (eic_video->dar_num*3 - eic_video->dar_den * 4)) /
                (double)((eic_video->dar_num * 3 + eic_video->dar_den * 4)));
        err2 = ((double)(2 * abs (eic_video->dar_num * 9 - eic_video->dar_den * 16)) /
                (double)((eic_video->dar_num * 9 + eic_video->dar_den * 16)));
        if (err1 < err2) {
          if (err1 * 100.0 < 3) {
            str = " {4:3}";
            err = 100.0 * err1;
          }
        }
        else {
          if (err2 * 100.0 < 3) {
            str = " {16:9}";
            err = 100.0 * err2;;
          }
        }
      }
    }

    snprintf (buffer, 4096, "%dx%d (%.3f:1) %s",
              eic_video->dar_num, eic_video->dar_den,
              (double)eic_video->dar_num / (double)eic_video->dar_den,
              str);
    str = strdup (buffer);
    ewl_text_text_set (EWL_TEXT (eic_ewl_video->dar), buffer);

    if (err < 0.0)
      snprintf (buffer, 4096,
                 "%s\n[*:*] means exact ratio\n{*:*} means approximative ratio",
                 str);
    else
      snprintf (buffer, 4096,
                 "%s\n[*:*] means exact ratio\n{*:*} means approximative ratio (%.1f %%)",
                 str, err);
    g_free (str);
/*     gtk_tooltips_set_tip (eic_gtk_video->tooltips, eic_gtk_video->dar, */
/*                           buffer, */
/*                           "Display aspect ratio of the video"); */
  }
}

static void
eic_ewl_audio_fill_cb (Ewl_Widget *widget, void *ev_data, void *data)
{
  char                   buffer[128];
  Eic_Ewl_Widgets       *eic_ewl;
  Eic_Prop              *eic;
  Eic_Audio_Prop        *eic_audio;
  Eic_Ewl_Audio_Widgets *eic_ewl_audio;
  GList                 *elt;

  eic_ewl = (Eic_Ewl_Widgets *)data;
  eic = eic_ewl->eic;
  if (!eic)
    return;

  eic_ewl_audio = eic_ewl->audio;
/*   elt = g_list_nth (eic->audio_prop, audio_index); */
  if (!elt)
    return;

  eic_audio = (Eic_Audio_Prop *)elt->data;
  if (!eic_audio)
    return;

  ewl_text_text_set (EWL_TEXT (eic_ewl_audio->decoder), eic_audio->decoder_info->name);
  snprintf (buffer, 128, "%d", eic_audio->samplerate);
  ewl_text_text_set (EWL_TEXT (eic_ewl_audio->samplerate), buffer);
  snprintf (buffer, 128, "%d", eic_audio->channels);
  ewl_text_text_set (EWL_TEXT (eic_ewl_audio->channels), buffer);
  snprintf (buffer, 128, "%u:%02u:%02u.%03u", EIC_TIME_ARGS (eic_audio->length_time));
  ewl_text_text_set (EWL_TEXT (eic_ewl_audio->length_time), buffer);
  snprintf (buffer, 128, "%lld", eic_audio->length_samples);
  ewl_text_text_set (EWL_TEXT (eic_ewl_audio->length_samples), buffer);
}

void
eic_ewl_fill (Eic_Ewl_Widgets *eic_ewl)
{
  char        container[128];
  Eic_Prop   *eic;
  char       *str;
  int         i;

  eic = eic_ewl->eic;
  if (!eic)
    return;

  ewl_text_text_set (EWL_TEXT (eic_ewl->filename), eic->filename);

  str = get_size_string (eic->size);
  ewl_text_text_set (EWL_TEXT (eic_ewl->filesize), str);
  free (str);

  if (eic->demuxer_info) {
    str = strchr (eic->demuxer_info->name, ' ');
    i = 127 < (str - eic->demuxer_info->name) ? 127 : str - eic->demuxer_info->name;
    strncpy (container, eic->demuxer_info->name, i);
    container[i] = '\0';
    ewl_text_text_set (EWL_TEXT (eic_ewl->container), container);
  }
  else {
    ewl_text_text_set (EWL_TEXT (eic_ewl->container), "no informations");
  }
  /* video */
  {
    Ewl_Model *model;
    Ewl_View  *view;

    eic_ewl_video_clear (eic_ewl->video, eic);

    model = ewl_model_new ();
    ewl_model_fetch_set (model, video_container_fetch);
    ewl_model_count_set (model, video_container_count_get);

    view = ewl_view_new ();
    ewl_view_constructor_set (view, video_container_name);
    ewl_view_assign_set (view, EWL_VIEW_ASSIGN (video_container_name_set));
    ewl_view_header_fetch_set (view, video_container_header_fetch);

    ewl_combo_model_set (EWL_COMBO (eic_ewl->video->list), model);
    ewl_combo_view_set (EWL_COMBO (eic_ewl->video->list), view);
    ewl_combo_data_set (EWL_COMBO (eic_ewl->video->list), eic);
    g_print ("combo done\n");
/*     eic_ewl_video_fill (eic_ewl, 0); */
  }

  /* audio */
  {
    Ewl_Model *model;
    Ewl_View  *view;

    eic_ewl_audio_clear (eic_ewl->audio, eic);

    model = ewl_model_new ();
    ewl_model_fetch_set (model, audio_container_fetch);
    ewl_model_count_set (model, audio_container_count_get);

    view = ewl_view_new ();
    ewl_view_constructor_set (view, audio_container_name);
    ewl_view_assign_set (view, EWL_VIEW_ASSIGN (audio_container_name_set));
    ewl_view_header_fetch_set (view, audio_container_header_fetch);

    ewl_combo_model_set (EWL_COMBO (eic_ewl->audio->list), model);
    ewl_combo_view_set (EWL_COMBO (eic_ewl->audio->list), view);
    ewl_combo_data_set (EWL_COMBO (eic_ewl->audio->list), eic);
/*   eic_ewl_audio_fill (eic_ewl, 0); */
  }
}

Ewl_Widget *
eic_ewl_video_frame (Eic_Ewl_Widgets *eic_ewl)
{
  Ewl_Widget *border;
  Ewl_Widget *table;
  Ewl_Widget *label;
  Eic_Ewl_Video_Widgets *video;

  if (!eic_ewl)
    return NULL;

  video = eic_ewl->video;

  border = ewl_border_new ();
  ewl_border_text_set (EWL_BORDER (border), "Video properties");

  table = ewl_table_new (2, 8, FALSE);
  ewl_container_child_append (EWL_CONTAINER (border), table);
  ewl_widget_show (table);

  label = ewl_label_new ();
  ewl_label_text_set (EWL_LABEL (label), "Stream:");
  ewl_table_add (EWL_TABLE (table), label, 1, 1, 1, 1);
  ewl_widget_show (label);

  ewl_table_add (EWL_TABLE (table), video->list, 2, 2, 1, 1);
  ewl_widget_show (video->list);

  label = ewl_label_new ();
  ewl_label_text_set (EWL_LABEL (label), "Decoder:");
  ewl_table_add (EWL_TABLE (table), label, 1, 1, 2, 2);
  ewl_widget_show (label);

  ewl_table_add (EWL_TABLE (table), video->decoder->name, 2, 2, 2, 2);
  ewl_widget_show (video->decoder->name);

  label = ewl_label_new ();
  ewl_label_text_set (EWL_LABEL (label), "Resolution:");
  ewl_table_add (EWL_TABLE (table), label, 1, 1, 3, 3);
  ewl_widget_show (label);

  ewl_table_add (EWL_TABLE (table), video->resolution, 2, 2, 3, 3);
  ewl_widget_show (video->resolution);

  label = ewl_label_new ();
  ewl_label_text_set (EWL_LABEL (label), "Frame rate:");
  ewl_table_add (EWL_TABLE (table), label, 1, 1, 4, 4);
  ewl_widget_show (label);

  ewl_table_add (EWL_TABLE (table), video->framerate, 2, 2, 4, 4);
  ewl_widget_show (video->framerate);

  label = ewl_label_new ();
  ewl_label_text_set (EWL_LABEL (label), "Pixel aspect ratio:");
  ewl_table_add (EWL_TABLE (table), label, 1, 1, 5, 5);
  ewl_widget_show (label);

  ewl_table_add (EWL_TABLE (table), video->par, 2, 2, 5, 5);
  ewl_widget_show (video->par);

  label = ewl_label_new ();
  ewl_label_text_set (EWL_LABEL (label), "Display aspect ratio:");
  ewl_table_add (EWL_TABLE (table), label, 1, 1, 6, 6);
  ewl_widget_show (label);

  ewl_table_add (EWL_TABLE (table), video->dar, 2, 2, 6, 6);
  ewl_widget_show (video->dar);

  label = ewl_label_new ();
  ewl_label_text_set (EWL_LABEL (label), "Duration:");
  ewl_table_add (EWL_TABLE (table), label, 1, 1, 7, 7);
  ewl_widget_show (label);

  ewl_table_add (EWL_TABLE (table), video->length_time, 2, 2, 7, 7);
  ewl_widget_show (video->length_time);

  label = ewl_label_new ();
  ewl_label_text_set (EWL_LABEL (label), "Frame count:");
  ewl_table_add (EWL_TABLE (table), label, 1, 1, 8, 8);
  ewl_widget_show (label);

  ewl_table_add (EWL_TABLE (table), video->length_frames, 2, 2, 8, 8);
  ewl_widget_show (video->length_frames);

  return border;
}

Ewl_Widget *
eic_ewl_audio_frame (Eic_Ewl_Widgets *eic_ewl)
{
  Ewl_Widget *border;
  Ewl_Widget *table;
  Ewl_Widget *label;
  Eic_Ewl_Audio_Widgets *audio;

  if (!eic_ewl)
    return NULL;

  audio = eic_ewl->audio;

  border = ewl_border_new ();
  ewl_border_text_set (EWL_BORDER (border), "Audio properties");

  table = ewl_table_new (2, 6, FALSE);
  ewl_container_child_append (EWL_CONTAINER (border), table);
  ewl_widget_show (table);

  label = ewl_label_new ();
  ewl_label_text_set (EWL_LABEL (label), "Stream:");
  ewl_table_add (EWL_TABLE (table), label, 1, 1, 1, 1);
  ewl_widget_show (label);

  ewl_table_add (EWL_TABLE (table), audio->list, 2, 2, 1, 1);
  ewl_widget_show (audio->list);

  label = ewl_label_new ();
  ewl_label_text_set (EWL_LABEL (label), "Decoder:");
  ewl_table_add (EWL_TABLE (table), label, 1, 1, 2, 2);
  ewl_widget_show (label);

  ewl_table_add (EWL_TABLE (table), audio->decoder->name, 2, 2, 2, 2);
  ewl_widget_show (audio->decoder->name);

  label = ewl_label_new ();
  ewl_label_text_set (EWL_LABEL (label), "Sample rate:");
  ewl_table_add (EWL_TABLE (table), label, 1, 1, 3, 3);
  ewl_widget_show (label);

  ewl_table_add (EWL_TABLE (table), audio->samplerate, 2, 2, 3, 3);
  ewl_widget_show (audio->samplerate);

  label = ewl_label_new ();
  ewl_label_text_set (EWL_LABEL (label), "Channels:");
  ewl_table_add (EWL_TABLE (table), label, 1, 1, 4, 4);
  ewl_widget_show (label);

  ewl_table_add (EWL_TABLE (table), audio->channels, 2, 2, 4, 4);
  ewl_widget_show (audio->channels);

  label = ewl_label_new ();
  ewl_label_text_set (EWL_LABEL (label), "Duration:");
  ewl_table_add (EWL_TABLE (table), label, 1, 1, 5, 5);
  ewl_widget_show (label);

  ewl_table_add (EWL_TABLE (table), audio->length_time, 2, 2, 5, 5);
  ewl_widget_show (audio->length_time);

  label = ewl_label_new ();
  ewl_label_text_set (EWL_LABEL (label), "Sample count:");
  ewl_table_add (EWL_TABLE (table), label, 1, 1, 6, 6);
  ewl_widget_show (label);

  ewl_table_add (EWL_TABLE (table), audio->length_samples, 2, 2, 6, 6);
  ewl_widget_show (audio->length_samples);

  return border;
}

Ewl_Widget *
eic_ewl_general_frame (Eic_Ewl_Widgets *eic_ewl)
{
  Ewl_Widget *border;
  Ewl_Widget *table;
  Ewl_Widget *label;

  border = ewl_border_new ();
  ewl_border_text_set (EWL_BORDER (border), "General");

  table = ewl_table_new (2, 3, FALSE);
  ewl_container_child_append (EWL_CONTAINER (border), table);
  ewl_widget_show (table);

  label = ewl_text_new ();
  ewl_text_text_set (EWL_TEXT (label), "Filename:");
  ewl_object_alignment_set (EWL_OBJECT (label), EWL_FLAG_ALIGN_LEFT);
  ewl_table_add (EWL_TABLE (table), label, 1, 1, 1, 1);
  ewl_widget_show (label);

  ewl_table_add (EWL_TABLE (table), eic_ewl->filename, 2, 2, 1, 1);
  ewl_widget_show (eic_ewl->filename);

  label = ewl_label_new ();
  ewl_label_text_set (EWL_LABEL (label), "File size:");
  ewl_object_alignment_set (EWL_OBJECT (label), EWL_FLAG_ALIGN_LEFT);
  ewl_table_add (EWL_TABLE (table), label, 1, 1, 2, 2);
  ewl_widget_show (label);

  ewl_table_add (EWL_TABLE (table), eic_ewl->filesize, 2, 2, 2, 2);
  ewl_widget_show (eic_ewl->filesize);

  label = ewl_label_new ();
  ewl_label_text_set (EWL_LABEL (label), "Container:");
  ewl_object_alignment_set (EWL_OBJECT (label), EWL_FLAG_ALIGN_LEFT);
  ewl_table_add (EWL_TABLE (table), label, 1, 1, 3, 3);
  ewl_widget_show (label);

  ewl_table_add (EWL_TABLE (table), eic_ewl->container, 2, 2, 3, 3);
  ewl_widget_show (eic_ewl->container);

  return border;
}

void
eic_ewl_main_window (Eic_Ewl_Widgets *eic_ewl)
{
  Ewl_Widget *win;
  Ewl_Widget *vbox;
  Ewl_Widget *menu_bar;
  Ewl_Widget *menu;
  Ewl_Widget *menu_item;
  Ewl_Widget *frame;
  Ewl_Widget *spacer;

  win = ewl_window_new ();
  ewl_window_title_set(EWL_WINDOW(win), "Eic - Enlightened Information Codec");
  ewl_callback_append (win, EWL_CALLBACK_DELETE_WINDOW,
                       EWL_CALLBACK_FUNCTION (eic_ewl_quit_cb), eic_ewl);

  vbox = ewl_vbox_new ();
  ewl_box_spacing_set (EWL_BOX (vbox), 6);
  ewl_box_homogeneous_set (EWL_BOX (vbox), FALSE);
  ewl_container_child_append (EWL_CONTAINER (win), vbox);
  ewl_widget_show (vbox);

  menu_bar = ewl_hmenubar_new ();
  ewl_object_fill_policy_set(EWL_OBJECT(menu_bar), EWL_FLAG_FILL_HFILL);
  ewl_container_child_append (EWL_CONTAINER (vbox), menu_bar);
  ewl_widget_show (menu_bar);

  /* File menu */
  menu_item = ewl_menu_new();
  ewl_button_label_set (EWL_BUTTON (menu_item), "File");
  ewl_object_fill_policy_set (EWL_OBJECT(menu_item), EWL_FLAG_FILL_NONE);
  ewl_container_child_append(EWL_CONTAINER(menu_bar), menu_item);
  ewl_widget_show (menu_item);

  menu = ewl_menu_item_new ();
  ewl_button_label_set (EWL_BUTTON (menu), "Open");
  ewl_container_child_append (EWL_CONTAINER (menu_item), menu);
  ewl_callback_append (menu, EWL_CALLBACK_CLICKED,
                       EWL_CALLBACK_FUNCTION (eic_ewl_open_cb), eic_ewl);
  ewl_widget_show (menu);

  menu = ewl_menu_item_new ();
  ewl_button_label_set (EWL_BUTTON (menu), "Quit");
  ewl_container_child_append (EWL_CONTAINER (menu_item), menu);
  ewl_callback_append (menu, EWL_CALLBACK_CLICKED,
                       EWL_CALLBACK_FUNCTION (eic_ewl_quit_cb), eic_ewl);
  ewl_widget_show (menu);

  spacer = ewl_spacer_new ();
  ewl_container_child_append(EWL_CONTAINER(menu_bar), spacer);
  ewl_widget_show (spacer);

  /* Help menu */
  menu_item = ewl_menu_new();
  ewl_button_label_set (EWL_BUTTON (menu_item), "Help");
  ewl_object_fill_policy_set (EWL_OBJECT(menu_item), EWL_FLAG_FILL_NONE);
  ewl_container_child_append(EWL_CONTAINER(menu_bar), menu_item);
  ewl_widget_show (menu_item);

  menu = ewl_menu_item_new ();
  ewl_button_label_set (EWL_BUTTON (menu), "About");
  ewl_container_child_append (EWL_CONTAINER (menu_item), menu);
  ewl_widget_show (menu);

  frame = eic_ewl_general_frame (eic_ewl);
  ewl_container_child_append (EWL_CONTAINER (vbox), frame);
  ewl_widget_show (frame);

  frame = eic_ewl_video_frame (eic_ewl);
  ewl_container_child_append (EWL_CONTAINER (vbox), frame);
  ewl_widget_show (frame);

  frame = eic_ewl_audio_frame (eic_ewl);
  ewl_container_child_append (EWL_CONTAINER (vbox), frame);
  ewl_widget_show (frame);

  ewl_widget_show (win);
}

int
main (int argc, char *argv[])
{
  Eic_Ewl_Widgets *eic_ewl;

  ewl_init (&argc, argv);

  eic_ewl = eic_ewl_init ();
  if (!eic_ewl) {
    ewl_main_quit ();
    return -1;
  }

  eic_ewl_main_window (eic_ewl);

  if (argc > 1) {
    eic_ewl->eic = eic_new (argv[1]);
    eic_fill (eic_ewl->eic);
    eic_ewl_fill (eic_ewl);
  }

  ewl_main ();

  return 0;
}

static void
eic_ewl_open_cb (Ewl_Widget *widget, void *ev_data, void *data)
{
  Ewl_Widget *fd;

  fd = ewl_filedialog_new ();
  ewl_callback_append (fd,
                       EWL_CALLBACK_VALUE_CHANGED,
                       eic_ewl_fd_response_cb, data);
  ewl_widget_show (fd);
}

static void
eic_ewl_fd_response_cb (Ewl_Widget *widget, void *ev_data, void *data)
{
  Ewl_Dialog_Event *ev;

  ev = (Ewl_Dialog_Event *)ev_data;
  printf ("response : %d\n", ev->response);

  switch (ev->response) {
  case EWL_STOCK_CANCEL:
    ewl_widget_destroy (widget);
  default: {
    Eic_Ewl_Widgets *eic_ewl;
    Ewl_Filedialog *fd;
    const char *path;
    char *filename;
    char *file;
    int l1;
    int l2;

    fd = EWL_FILEDIALOG (widget);
    path = ewl_filedialog_directory_get (fd);
    filename = ewl_filedialog_selected_file_get (fd);
    if (!path || !filename)
      return;

    l1 = strlen (path);
    l2 = strlen (filename);
    file = (char *)malloc (sizeof (char) * (l1 + l2 + 2));
    memcpy (file, path, l1);
    file[l1] = '/';
    memcpy (file + l1 + 1, filename, l2);
    file[l1 + l2 + 1] = '\0';
    printf ("file : %s\n", file);
    ewl_widget_destroy (widget);

    eic_ewl = (Eic_Ewl_Widgets *)data;
    eic_ewl->eic = eic_new (file);
    eic_fill (eic_ewl->eic);
    eic_ewl_fill (eic_ewl);
    free (file);
  }
  }
}

static void
eic_ewl_quit_cb (Ewl_Widget *widget, void *ev_data, void *data)
{
  Eic_Ewl_Widgets *eic_ewl;

  eic_ewl = (Eic_Ewl_Widgets *)data;
  eic_free (eic_ewl->eic);
  eic_ewl_free (eic_ewl);
  ewl_main_quit();
}

/* Combo model functions */
static void *
video_container_fetch (void *data, unsigned int row, unsigned int column)
{
  Eic_Prop *eic;
  GList    *list;
  Eic_Video_Prop *vp;
  gchar           buffer[1024];

  eic = (Eic_Prop *)data;
  list = g_list_nth (eic->video_prop, row);

  vp = (Eic_Video_Prop *)list->data;
  g_snprintf (buffer, 1024, "%s (%d/%d)",
              vp->mimetype,
              g_list_position (g_list_first (list), list) + 1,
              g_list_length (g_list_first (list)));

  return strdup (buffer);
}

static int
video_container_count_get (void *data)
{
  Eic_Prop *eic;

  eic = (Eic_Prop *)data;

  return g_list_length (eic->video_prop);
}

static void *
audio_container_fetch (void *data, unsigned int row, unsigned int column)
{
  Eic_Prop *eic;
  GList    *list;
  Eic_Audio_Prop *vp;
  gchar           buffer[1024];

  eic = (Eic_Prop *)data;
  list = g_list_nth (eic->audio_prop, row);

  vp = (Eic_Audio_Prop *)list->data;
  g_snprintf (buffer, 1024, "%s (%d/%d)",
              vp->mimetype,
              g_list_position (g_list_first (list), list) + 1,
              g_list_length (g_list_first (list)));

  return strdup (buffer);
}

static int
audio_container_count_get (void *data)
{
  Eic_Prop *eic;

  eic = (Eic_Prop *)data;

  return g_list_length (eic->audio_prop);
}

/* Combo view functions */
static Ewl_Widget *video_container_name ()
{
  return ewl_label_new ();
}

static void video_container_name_set (Ewl_Widget *w, void *data)
{
  ewl_label_text_set (EWL_LABEL (w), (char *)data);
}

static Ewl_Widget *video_container_header_fetch (void *data, int column)
{
  Ewl_Widget *label;
  
  label = ewl_label_new ();
  ewl_label_text_set (EWL_LABEL (label), "toto");

  return label;
}

static Ewl_Widget *audio_container_name ()
{
  return ewl_label_new ();
}

static void audio_container_name_set (Ewl_Widget *w, void *data)
{
  ewl_label_text_set (EWL_LABEL (w), (char *)data);
}

static Ewl_Widget *audio_container_header_fetch (void *data, int column)
{
  Ewl_Widget *label;
  
  label = ewl_label_new ();
  ewl_label_text_set (EWL_LABEL (label), "toto");

  return label;
}
