#include <stdlib.h>

#include <gtk/gtk.h>

#include "../lib/Eic.h"


#define EIC_NAME "E Stream Information"
#define EIC_VERSION "0.1" /* Should take that from config.h */
#define EIC_COPYRIGHT "Copyright 2006 Vincent Torri"
#define EIC_COMMENTS "Retrieve the informations of a multimedia file\nusing the Gstreamer 0.10 library"
#define EIC_LICENSE "GPL"
#define EIC_WEBSITE "http://www.iecn.u-nancy.fr/~torri"
#define EIC_AUTHOR0 "Vincent Torri"

typedef struct Eic_Gtk_Factory_Widgets_ Eic_Gtk_Factory_Widgets;
typedef struct Eic_Gtk_Audio_Widgets_ Eic_Gtk_Audio_Widgets;
typedef struct Eic_Gtk_Video_Widgets_ Eic_Gtk_Video_Widgets;
typedef struct Eic_Gtk_Widgets_ Eic_Gtk_Widgets;

struct Eic_Gtk_Factory_Widgets_
{
  GtkWidget *name;
  GtkWidget *description;
  GtkWidget *class;
};

struct Eic_Gtk_Audio_Widgets_
{
  GtkWidget              *list;
  GtkWidget              *decoder;
  GtkWidget              *samplerate;
  GtkWidget              *channels;
  GtkWidget              *length_time;
  GtkWidget              *length_samples;
  GtkTooltips            *tooltips;
};

struct Eic_Gtk_Video_Widgets_
{
  GtkWidget              *list;
  GtkWidget              *decoder;
  GtkWidget              *resolution;
  GtkWidget              *framerate;
  GtkWidget              *par;
  GtkWidget              *dar;
  GtkWidget              *length_time;
  GtkWidget              *length_frames;
  GtkTooltips            *tooltips;
};

struct Eic_Gtk_Widgets_
{
  Eic_Prop                *eic;

  GtkWidget              *filename;
  GtkWidget              *filesize;
  GtkWidget              *container;
  GtkTooltips            *tooltips;

  Eic_Gtk_Video_Widgets   *video;
  Eic_Gtk_Audio_Widgets   *audio;
};

static gboolean eic_gtk_delete_cb (GtkWidget *widget,
                                   GdkEvent  *event,
                                   gpointer   data);
static void eic_gtk_menu_quit_cb (GtkMenuItem *menu_item,
                                  gpointer     data);
static void eic_gtk_about_cb    (GtkMenuItem *menu_item,
                                 gpointer     data);
static void eic_gtk_video_fill_cb (GtkComboBox *combo,
                                   gpointer     data);
static void eic_gtk_audio_fill_cb (GtkComboBox *combo,
                                   gpointer     data);
static void eic_gtk_open_cb (GtkWidget *widget,
                             gpointer   data);
static void eic_gtk_fd_response_cb (GtkWidget *widget,
                                    gint       arg1,
                                    gpointer   data);

void
eic_gtk_factory_info_attach (GtkTooltips      *tooltips,
                             GtkWidget        *parent,
                             Eic_Factory_Info *factory)
{
  char *str;
  char  buffer[4096];
  int   nb_authors;
  int   index;

  if (!factory)
    return;

  for (nb_authors = 0 ; factory->authors[nb_authors] ; nb_authors++) {}
  if (nb_authors == 1)
    str = "Author........: ";
  else
    str = "Authors.......: ";

  g_snprintf (buffer, 4096,
             "Name..........: %s\nDescription...: %s\nClass.........: %s\n%s%s",
             factory->name,
             factory->description,
             factory->class,
             str,
             factory->authors[0]);
  for (index = 1 ; index < nb_authors ; index++) {
    g_snprintf (buffer, 4096, "%s%s%s\n",
               buffer,
               "\n                    ",
               factory->authors[index]);
  }
  gtk_tooltips_set_tip (tooltips, parent, buffer, "");
}

Eic_Gtk_Video_Widgets *
eic_gtk_video_init (Eic_Gtk_Widgets *eic_gtk)
{
  Eic_Gtk_Video_Widgets *video;

  video = (Eic_Gtk_Video_Widgets *)g_malloc (sizeof (Eic_Gtk_Video_Widgets));
  if (!video)
    return NULL;

  video->list = gtk_combo_box_new_text ();
  g_signal_connect (G_OBJECT (video->list), "changed",
		    G_CALLBACK (eic_gtk_video_fill_cb), eic_gtk);
  video->decoder = gtk_entry_new ();
  video->resolution = gtk_entry_new ();
  video->framerate = gtk_entry_new ();
  video->par = gtk_entry_new ();
  video->dar = gtk_entry_new ();
  video->length_time = gtk_entry_new ();
  video->length_frames = gtk_entry_new ();
  video->tooltips = gtk_tooltips_new ();

  return video;
}

void
eic_gtk_video_clear (Eic_Gtk_Video_Widgets *video, Eic_Prop *eic)
{
  int i;

  for (i = 0; i < g_list_length (eic->video_prop); i++)
    gtk_combo_box_remove_text (GTK_COMBO_BOX (video->list), i);
  gtk_entry_set_text (GTK_ENTRY (video->decoder), "");
  gtk_entry_set_text (GTK_ENTRY (video->resolution), "");
  gtk_entry_set_text (GTK_ENTRY (video->framerate), "");
  gtk_entry_set_text (GTK_ENTRY (video->par), "");
  gtk_entry_set_text (GTK_ENTRY (video->dar), "");
  gtk_entry_set_text (GTK_ENTRY (video->length_time), "");
  gtk_entry_set_text (GTK_ENTRY (video->length_frames), "");
}

Eic_Gtk_Audio_Widgets *
eic_gtk_audio_init (Eic_Gtk_Widgets *eic_gtk)
{
  Eic_Gtk_Audio_Widgets *audio;

  audio = (Eic_Gtk_Audio_Widgets *)g_malloc (sizeof (Eic_Gtk_Audio_Widgets));
  if (!audio)
    return NULL;

  audio->list = gtk_combo_box_new_text ();
  g_signal_connect (G_OBJECT (audio->list), "changed",
		    G_CALLBACK (eic_gtk_audio_fill_cb), eic_gtk);
  audio->decoder = gtk_entry_new ();
  audio->samplerate = gtk_entry_new ();
  audio->channels = gtk_entry_new ();
  audio->length_time = gtk_entry_new ();
  audio->length_samples = gtk_entry_new ();
  audio->tooltips = gtk_tooltips_new ();

  return audio;
}

void
eic_gtk_audio_clear (Eic_Gtk_Audio_Widgets *audio, Eic_Prop *eic)
{
  int i;

  for (i = 0; i < g_list_length (eic->audio_prop); i++)
    gtk_combo_box_remove_text (GTK_COMBO_BOX (audio->list), i);
  gtk_entry_set_text (GTK_ENTRY (audio->decoder), "");
  gtk_entry_set_text (GTK_ENTRY (audio->samplerate), "");
  gtk_entry_set_text (GTK_ENTRY (audio->channels), "");
  gtk_entry_set_text (GTK_ENTRY (audio->length_time), "");
  gtk_entry_set_text (GTK_ENTRY (audio->length_samples), "");
}

Eic_Gtk_Widgets *
eic_gtk_init ()
{
  Eic_Gtk_Widgets *eic_gtk;

  eic_gtk = (Eic_Gtk_Widgets *)g_malloc (sizeof (Eic_Gtk_Widgets));
  if (!eic_gtk)
    return NULL;

  eic_gtk->eic = NULL;
  eic_gtk->filename = gtk_entry_new ();
  eic_gtk->filesize = gtk_entry_new ();
  eic_gtk->container = gtk_entry_new ();
  eic_gtk->tooltips = gtk_tooltips_new ();
  eic_gtk->video = eic_gtk_video_init (eic_gtk);
  if (!eic_gtk->video) {
    g_free (eic_gtk);
    return NULL;
  }
  eic_gtk->audio = eic_gtk_audio_init (eic_gtk);
  if (!eic_gtk->audio) {
    g_free (eic_gtk->video);
    g_free (eic_gtk);
    return NULL;
  }

  return eic_gtk;
}

void
eic_gtk_free (Eic_Gtk_Widgets *eic_gtk)
{
  if (!eic_gtk)
    return;

  if (eic_gtk->video)
    g_free (eic_gtk->video);

  if (eic_gtk->audio)
    g_free (eic_gtk->audio);

  g_free (eic_gtk);
}

static void
eic_gtk_video_fill_cb (GtkComboBox *combo, gpointer data)
{
  char                   buffer[4096];
  Eic_Gtk_Widgets       *eic_gtk;
  Eic_Prop              *eic;
  Eic_Video_Prop        *eic_video;
  Eic_Gtk_Video_Widgets *eic_gtk_video;
  GList                 *elt;
  char                  *str;
  double                 err1;
  double                 err2;
  double                 err;

  eic_gtk = (Eic_Gtk_Widgets *)data;
  eic = eic_gtk->eic;
  if (!eic)
    return;

  eic_gtk_video = eic_gtk->video;
  elt = g_list_nth (eic->video_prop, gtk_combo_box_get_active (combo));
  if (!elt)
    return;

  eic_video = (Eic_Video_Prop *)elt->data;
  if (!eic_video)
    return;

  gtk_entry_set_text (GTK_ENTRY (eic_gtk_video->decoder), eic_video->decoder_info->name);
  eic_gtk_factory_info_attach (eic_gtk_video->tooltips, eic_gtk_video->decoder, eic_video->decoder_info);
  g_snprintf (buffer, 128, "%dx%d", eic_video->width, eic_video->height);
  gtk_entry_set_text (GTK_ENTRY (eic_gtk_video->resolution), buffer);
  g_snprintf (buffer, 128, "%d/%d", eic_video->framerate_num, eic_video->framerate_den);
  gtk_entry_set_text (GTK_ENTRY (eic_gtk_video->framerate), buffer);
  g_snprintf (buffer, 128, "%dx%d", eic_video->par_num, eic_video->par_den);
  gtk_entry_set_text (GTK_ENTRY (eic_gtk_video->par), buffer);
  g_snprintf (buffer, 128, "%u:%02u:%02u.%03u", EIC_TIME_ARGS (eic_video->length_time));
  gtk_entry_set_text (GTK_ENTRY (eic_gtk_video->length_time), buffer);
  g_snprintf (buffer, 128, "%lld", eic_video->length_frames);
  gtk_entry_set_text (GTK_ENTRY (eic_gtk_video->length_frames), buffer);

  if ((eic_video->dar_num > 0) && (eic_video->dar_den)) {
    err = -1.0;
    str = "";
    if (eic_video->dar_num*9 == eic_video->dar_den*16)
      str = " [16:9]";
    else {
      if (eic_video->dar_num*3 == eic_video->dar_den*4)
        str = " [4:3]";
      else {
        err1 = ((double)(2 * abs (eic_video->dar_num*3 - eic_video->dar_den * 4)) /
                (double)((eic_video->dar_num * 3 + eic_video->dar_den * 4)));
        err2 = ((double)(2 * abs (eic_video->dar_num * 9 - eic_video->dar_den * 16)) /
                (double)((eic_video->dar_num * 9 + eic_video->dar_den * 16)));
        if (err1 < err2) {
          if (err1 * 100.0 < 3) {
            str = " {4:3}";
            err = 100.0 * err1;
          }
        }
        else {
          if (err2 * 100.0 < 3) {
            str = " {16:9}";
            err = 100.0 * err2;;
          }
        }
      }
    }

    g_snprintf (buffer, 4096, "%dx%d (%.3f:1) %s",
               eic_video->dar_num, eic_video->dar_den,
               (gdouble)eic_video->dar_num / (gdouble)eic_video->dar_den,
               str);
    str = g_strdup (buffer);
    gtk_entry_set_text (GTK_ENTRY (eic_gtk_video->dar), str);

    if (err < 0.0)
      g_snprintf (buffer, 4096,
                 "%s\n[*:*] means exact ratio\n{*:*} means approximative ratio",
                 str);
    else
      g_snprintf (buffer, 4096,
                 "%s\n[*:*] means exact ratio\n{*:*} means approximative ratio (%.1f %%)",
                 str, err);
    g_free (str);
    gtk_tooltips_set_tip (eic_gtk_video->tooltips, eic_gtk_video->dar,
                          buffer,
                          "Display aspect ratio of the video");
  }
}

static void
eic_gtk_audio_fill_cb (GtkComboBox *combo, gpointer data)
{
  char                   str[128];
  Eic_Gtk_Widgets       *eic_gtk;
  Eic_Prop              *eic;
  Eic_Audio_Prop        *eic_audio;
  Eic_Gtk_Audio_Widgets *eic_gtk_audio;
  GList                 *elt;

  eic_gtk = (Eic_Gtk_Widgets *)data;
  eic = eic_gtk->eic;
  if (!eic)
    return;

  eic_gtk_audio = eic_gtk->audio;
  elt = g_list_nth (eic->audio_prop, gtk_combo_box_get_active (combo));
  if (!elt)
    return;

  eic_audio = (Eic_Audio_Prop *)elt->data;
  if (!eic_audio)
    return;

  gtk_entry_set_text (GTK_ENTRY (eic_gtk_audio->decoder), eic_audio->decoder_info->name);
  eic_gtk_factory_info_attach (eic_gtk_audio->tooltips, eic_gtk_audio->decoder, eic_audio->decoder_info);
  g_snprintf (str, 128, "%d", eic_audio->samplerate);
  gtk_entry_set_text (GTK_ENTRY (eic_gtk_audio->samplerate), str);
  g_snprintf (str, 128, "%d", eic_audio->channels);
  gtk_entry_set_text (GTK_ENTRY (eic_gtk_audio->channels), str);
  g_snprintf (str, 128, "%u:%02u:%02u.%03u", EIC_TIME_ARGS (eic_audio->length_time));
  gtk_entry_set_text (GTK_ENTRY (eic_gtk_audio->length_time), str);
  g_snprintf (str, 128, "%lld", eic_audio->length_samples);
  gtk_entry_set_text (GTK_ENTRY (eic_gtk_audio->length_samples), str);
}

void
eic_gtk_fill (Eic_Gtk_Widgets *eic_gtk)
{
  Eic_Prop *eic;
  GList    *list;
  char     *str;
  char    **container;

  eic = eic_gtk->eic;
  if (!eic)
    return;

  gtk_entry_set_text (GTK_ENTRY (eic_gtk->filename), eic->filename);
  gtk_tooltips_set_tip (eic_gtk->tooltips, eic_gtk->filename,
                        eic->filename,
                        "Name of the file");

  str = get_size_string (eic->size);
  gtk_entry_set_text (GTK_ENTRY (eic_gtk->filesize), str);
  gtk_tooltips_set_tip (eic_gtk->tooltips, eic_gtk->filesize,
                        str,
                        "Size of the file");
  g_free (str);

  if (eic->demuxer_info) {
    container = g_strsplit (eic->demuxer_info->name, " ", 1);
    gtk_entry_set_text (GTK_ENTRY (eic_gtk->container), container[0]);
    g_strfreev (container);
    eic_gtk_factory_info_attach (eic_gtk->tooltips, eic_gtk->container, eic->demuxer_info);
  }
  else {
    gtk_entry_set_text (GTK_ENTRY (eic_gtk->container), "no informations");
  }

  /* video */
  eic_gtk_video_clear (eic_gtk->video, eic);
  list = g_list_first (eic->video_prop);
  for (; list ; list = g_list_next (list)) {
    Eic_Video_Prop *vp;
    gchar           buffer[1024];

    vp = (Eic_Video_Prop *)list->data;
    g_snprintf (buffer, 1024, "%s (%d/%d)",
                vp->mimetype,
                g_list_position (g_list_first (list), list) + 1,
                g_list_length (g_list_first (list)));
    gtk_combo_box_append_text (GTK_COMBO_BOX (eic_gtk->video->list),
                               buffer);
  }

  /* audio */
  eic_gtk_audio_clear (eic_gtk->audio, eic);
  list = g_list_first (eic->audio_prop);
  for (; list ; list = g_list_next (list)) {
    Eic_Audio_Prop *ap;
    gchar           buffer[1024];

    ap = (Eic_Audio_Prop *)list->data;
    g_snprintf (buffer, 1024, "%s (%d/%d)",
                ap->mimetype,
                g_list_position (g_list_first (list), list) + 1,
                g_list_length (g_list_first (list)));
    gtk_combo_box_append_text (GTK_COMBO_BOX (eic_gtk->audio->list),
                               buffer);
  }

  gtk_combo_box_set_active (GTK_COMBO_BOX (eic_gtk->video->list), 0);
  gtk_combo_box_set_active (GTK_COMBO_BOX (eic_gtk->audio->list), 0);
}

GtkWidget *
eic_gtk_video_frame (Eic_Gtk_Widgets *eic_gtk)
{
  GtkWidget *frame;
  GtkWidget *table;
  GtkWidget *label;
  Eic_Gtk_Video_Widgets *video;

  if (!eic_gtk)
    return NULL;

  video = eic_gtk->video;

  frame = gtk_frame_new ("Video properties");

  table = gtk_table_new (2, 8, FALSE);
  gtk_table_set_homogeneous (GTK_TABLE (table), FALSE);
  gtk_table_set_row_spacings (GTK_TABLE (table), 6);
  gtk_table_set_col_spacings (GTK_TABLE (table), 6);
  gtk_container_set_border_width (GTK_CONTAINER (table), 6);
  gtk_container_add (GTK_CONTAINER (frame), table);
  gtk_widget_show (table);

  label = gtk_label_new ("Stream:");
  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
  gtk_table_attach_defaults (GTK_TABLE (table), label, 0, 1, 0, 1);
  gtk_widget_show (label);

  gtk_table_attach_defaults (GTK_TABLE (table), video->list, 1, 2, 0, 1);
  gtk_widget_show (video->list);

  label = gtk_label_new ("Decoder:");
  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
  gtk_table_attach_defaults (GTK_TABLE (table), label, 0, 1, 1, 2);
  gtk_widget_show (label);

  gtk_table_attach_defaults (GTK_TABLE (table), video->decoder, 1, 2, 1, 2);
  gtk_widget_show (video->decoder);

  label = gtk_label_new ("Resolution:");
  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
  gtk_table_attach_defaults (GTK_TABLE (table), label, 0, 1, 2, 3);
  gtk_widget_show (label);

  gtk_table_attach_defaults (GTK_TABLE (table), video->resolution, 1, 2, 2, 3);
  gtk_widget_show (video->resolution);

  label = gtk_label_new ("Frame rate:");
  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
  gtk_table_attach_defaults (GTK_TABLE (table), label, 0, 1, 3, 4);
  gtk_widget_show (label);

  gtk_table_attach_defaults (GTK_TABLE (table), video->framerate, 1, 2, 3, 4);
  gtk_widget_show (video->framerate);

  label = gtk_label_new ("Pixel aspect ratio:");
  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
  gtk_table_attach_defaults (GTK_TABLE (table), label, 0, 1, 4, 5);
  gtk_widget_show (label);

  gtk_table_attach_defaults (GTK_TABLE (table), video->par, 1, 2, 4, 5);
  gtk_widget_show (video->par);

  label = gtk_label_new ("Display aspect ratio:");
  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
  gtk_table_attach_defaults (GTK_TABLE (table), label, 0, 1, 5, 6);
  gtk_widget_show (label);

  gtk_table_attach_defaults (GTK_TABLE (table), video->dar, 1, 2, 5, 6);
  gtk_widget_show (video->dar);

  label = gtk_label_new ("Duration:");
  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
  gtk_table_attach_defaults (GTK_TABLE (table), label, 0, 1, 6, 7);
  gtk_widget_show (label);

  gtk_table_attach_defaults (GTK_TABLE (table), video->length_time, 1, 2, 6, 7);
  gtk_widget_show (video->length_time);

  label = gtk_label_new ("Frame count:");
  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
  gtk_table_attach_defaults (GTK_TABLE (table), label, 0, 1, 7, 8);
  gtk_widget_show (label);

  gtk_table_attach_defaults (GTK_TABLE (table), video->length_frames, 1, 2, 7, 8);
  gtk_widget_show (video->length_frames);

  return frame;
}

GtkWidget *
eic_gtk_audio_frame (Eic_Gtk_Widgets *eic_gtk)
{
  GtkWidget *frame;
  GtkWidget *table;
  GtkWidget *label;
  Eic_Gtk_Audio_Widgets *audio;

  if (!eic_gtk)
    return NULL;

  audio = eic_gtk->audio;

  frame = gtk_frame_new ("Audio properties");

  table = gtk_table_new (2, 6, FALSE);
  gtk_table_set_homogeneous (GTK_TABLE (table), FALSE);
  gtk_table_set_row_spacings (GTK_TABLE (table), 6);
  gtk_table_set_col_spacings (GTK_TABLE (table), 6);
  gtk_container_set_border_width (GTK_CONTAINER (table), 6);
  gtk_container_add (GTK_CONTAINER (frame), table);
  gtk_widget_show (table);

  label = gtk_label_new ("Stream:");
  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
  gtk_table_attach_defaults (GTK_TABLE (table), label, 0, 1, 0, 1);
  gtk_widget_show (label);

  gtk_table_attach_defaults (GTK_TABLE (table), audio->list, 1, 2, 0, 1);
  gtk_widget_show (audio->list);

  label = gtk_label_new ("Decoder:");
  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
  gtk_table_attach_defaults (GTK_TABLE (table), label, 0, 1, 1, 2);
  gtk_widget_show (label);

  gtk_table_attach_defaults (GTK_TABLE (table), audio->decoder, 1, 2, 1, 2);
  gtk_widget_show (audio->decoder);

  label = gtk_label_new ("Sample rate:");
  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
  gtk_table_attach_defaults (GTK_TABLE (table), label, 0, 1, 2, 3);
  gtk_widget_show (label);

  gtk_table_attach_defaults (GTK_TABLE (table), audio->samplerate, 1, 2, 2, 3);
  gtk_widget_show (audio->samplerate);

  label = gtk_label_new ("Channels:");
  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
  gtk_table_attach_defaults (GTK_TABLE (table), label, 0, 1, 3, 4);
  gtk_widget_show (label);

  gtk_table_attach_defaults (GTK_TABLE (table), audio->channels, 1, 2, 3, 4);
  gtk_widget_show (audio->channels);

  label = gtk_label_new ("Duration:");
  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
  gtk_table_attach_defaults (GTK_TABLE (table), label, 0, 1, 4, 5);
  gtk_widget_show (label);

  gtk_table_attach_defaults (GTK_TABLE (table), audio->length_time, 1, 2, 4, 5);
  gtk_widget_show (audio->length_time);

  label = gtk_label_new ("Sample count:");
  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
  gtk_table_attach_defaults (GTK_TABLE (table), label, 0, 1, 5, 6);
  gtk_widget_show (label);

  gtk_table_attach_defaults (GTK_TABLE (table), audio->length_samples, 1, 2, 5, 6);
  gtk_widget_show (audio->length_samples);

  return frame;
}

GtkWidget *
eic_gtk_general_frame (Eic_Gtk_Widgets *eic_gtk)
{
  GtkWidget *frame;
  GtkWidget *table;
  GtkWidget *label;

  frame = gtk_frame_new ("General");

  table = gtk_table_new (2, 3, FALSE);
  gtk_table_set_homogeneous (GTK_TABLE (table), FALSE);
  gtk_table_set_row_spacings (GTK_TABLE (table), 6);
  gtk_table_set_col_spacings (GTK_TABLE (table), 6);
  gtk_container_set_border_width (GTK_CONTAINER (table), 6);
  gtk_container_add (GTK_CONTAINER (frame), table);
  gtk_widget_show (table);

  label = gtk_label_new ("Filename:");
  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
  gtk_table_attach_defaults (GTK_TABLE (table), label, 0, 1, 0, 1);
  gtk_widget_show (label);

  gtk_table_attach_defaults (GTK_TABLE (table), eic_gtk->filename, 1, 2, 0, 1);
  gtk_widget_show (eic_gtk->filename);

  label = gtk_label_new ("File size:");
  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
  gtk_table_attach_defaults (GTK_TABLE (table), label, 0, 1, 1, 2);
  gtk_widget_show (label);

  gtk_table_attach_defaults (GTK_TABLE (table), eic_gtk->filesize, 1, 2, 1, 2);
  gtk_widget_show (eic_gtk->filesize);

  label = gtk_label_new ("Container:");
  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
  gtk_table_attach_defaults (GTK_TABLE (table), label, 0, 1, 2, 3);
  gtk_widget_show (label);

  gtk_table_attach_defaults (GTK_TABLE (table), eic_gtk->container, 1, 2, 2, 3);
  gtk_widget_show (eic_gtk->container);

  return frame;
}

void
eic_gtk_main_window (Eic_Gtk_Widgets *eic_gtk)
{
  GtkWidget *win;
  GtkWidget *vbox;
  GtkWidget *vbox2;
  GtkWidget *menu_bar;
  GtkWidget *menu;
  GtkWidget *menu_item;
  GtkWidget *frame;

  win = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title (GTK_WINDOW (win),
                        "Eic - Enlightened Information Codec");
  g_object_set_data (G_OBJECT (win), "eic-gtk", eic_gtk);
  g_signal_connect(G_OBJECT(win), "delete-event", G_CALLBACK(eic_gtk_delete_cb), eic_gtk);

  vbox = gtk_vbox_new (FALSE, 0);
  gtk_container_add (GTK_CONTAINER (win), vbox);
  gtk_widget_show (vbox);

  menu_bar = gtk_menu_bar_new();
  gtk_box_pack_start(GTK_BOX(vbox), menu_bar, FALSE, FALSE, 0);
  gtk_widget_show (menu_bar);

  /* File menu */
  menu_item = gtk_menu_item_new_with_label ("File");
  gtk_menu_shell_append (GTK_MENU_SHELL (menu_bar), menu_item);
  gtk_widget_show (menu_item);

  menu = gtk_menu_new();
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (menu_item), menu);
  gtk_widget_show (menu);

  menu_item = gtk_image_menu_item_new_from_stock ("gtk-open", NULL);
  gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
  g_signal_connect (G_OBJECT (menu_item), "activate",
		    G_CALLBACK (eic_gtk_open_cb), win);
  gtk_widget_show (menu_item);

  menu_item = gtk_separator_menu_item_new ();
  gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
  gtk_widget_show (menu_item);

  menu_item = gtk_image_menu_item_new_from_stock ("gtk-quit", NULL);
  gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
  g_signal_connect (G_OBJECT (menu_item), "activate",
		    G_CALLBACK (eic_gtk_menu_quit_cb), eic_gtk);
  gtk_widget_show (menu_item);

  /* Help menu */
  menu_item = gtk_menu_item_new_with_label("Help");
  gtk_menu_item_set_right_justified (GTK_MENU_ITEM (menu_item), TRUE);
  gtk_menu_shell_append (GTK_MENU_SHELL (menu_bar), menu_item);
  gtk_widget_show (menu_item);

  menu = gtk_menu_new();
  gtk_menu_item_set_submenu (GTK_MENU_ITEM(menu_item), menu);
  gtk_widget_show (menu);

  menu_item = gtk_image_menu_item_new_from_stock ("gtk-about", NULL);
  gtk_menu_shell_append(GTK_MENU_SHELL(menu), menu_item);
  g_signal_connect (G_OBJECT (menu_item), "activate",
		    G_CALLBACK (eic_gtk_about_cb), win);
  gtk_widget_show (menu_item);

  vbox2 = gtk_vbox_new (FALSE, 6);
  gtk_container_set_border_width (GTK_CONTAINER (vbox2), 6);
  gtk_container_add (GTK_CONTAINER (vbox), vbox2);
  gtk_widget_show (vbox2);

  frame = eic_gtk_general_frame (eic_gtk);
  gtk_box_pack_start (GTK_BOX (vbox2), frame, TRUE, TRUE, 0);
  gtk_widget_show (frame);

  frame = eic_gtk_video_frame (eic_gtk);
  gtk_box_pack_start (GTK_BOX (vbox2), frame, TRUE, TRUE, 0);
  gtk_widget_show (frame);

  frame = eic_gtk_audio_frame (eic_gtk);
  gtk_box_pack_start (GTK_BOX (vbox2), frame, TRUE, TRUE, 0);
  gtk_widget_show (frame);

  gtk_widget_show (win);
}

int
main (int argc, char *argv[])
{
  Eic_Gtk_Widgets *eic_gtk;

  gtk_init (&argc, &argv);

  eic_gtk = eic_gtk_init ();
  if (!eic_gtk) {
    gtk_main_quit ();
    return -1;
  }

  eic_gtk_main_window (eic_gtk);

  if (argc > 1) {
    eic_gtk->eic = eic_new (argv[1]);
    eic_fill (eic_gtk->eic);
    eic_gtk_fill (eic_gtk);
  }

  gtk_main ();

  return 0;
}

static gboolean
eic_gtk_delete_cb (GtkWidget *widget,
                   GdkEvent  *event,
                   gpointer   data)
{
  Eic_Gtk_Widgets *eic_gtk;

  eic_gtk = (Eic_Gtk_Widgets *)data;
  eic_free (eic_gtk->eic);
  eic_gtk_free (eic_gtk);

  gtk_main_quit();

  return FALSE;
}

static void
eic_gtk_menu_quit_cb (GtkMenuItem *menu_item,
                      gpointer     data)
{
  Eic_Gtk_Widgets *eic_gtk;

  eic_gtk = (Eic_Gtk_Widgets *)data;
  eic_free (eic_gtk->eic);
  eic_gtk_free (eic_gtk);

  gtk_main_quit();
}

static void
eic_gtk_about_cb (GtkMenuItem *menu_item,
                  gpointer     data)
{
  GtkWidget   *dialog;
  const gchar *authors[] = { EIC_AUTHOR0, NULL };

  dialog = gtk_about_dialog_new ();
  gtk_about_dialog_set_name (GTK_ABOUT_DIALOG (dialog), EIC_NAME);
  gtk_about_dialog_set_version (GTK_ABOUT_DIALOG (dialog), EIC_VERSION);
  gtk_about_dialog_set_copyright (GTK_ABOUT_DIALOG (dialog), EIC_COPYRIGHT);
  gtk_about_dialog_set_comments (GTK_ABOUT_DIALOG (dialog), EIC_COMMENTS);
  gtk_about_dialog_set_license (GTK_ABOUT_DIALOG (dialog), EIC_LICENSE);
  gtk_about_dialog_set_website (GTK_ABOUT_DIALOG (dialog), EIC_WEBSITE);
  gtk_about_dialog_set_authors (GTK_ABOUT_DIALOG (dialog), authors);

  gtk_dialog_run (GTK_DIALOG (dialog));
}

static void
eic_gtk_open_cb (GtkWidget *widget,
                 gpointer   data)
{
  GtkWidget *fd;

  fd = gtk_file_chooser_dialog_new ("Open File",
                                    GTK_WINDOW (data),
                                    GTK_FILE_CHOOSER_ACTION_OPEN,
                                    GTK_STOCK_CANCEL,
                                    GTK_RESPONSE_CANCEL,
                                    GTK_STOCK_OPEN,
                                    GTK_RESPONSE_ACCEPT,
                                    NULL);
  gtk_window_set_transient_for (GTK_WINDOW (fd),
                                GTK_WINDOW (data));  /* doesn't seem to work. Anyway, it's ugly */
  g_signal_connect (G_OBJECT (fd), "response",
                    G_CALLBACK (eic_gtk_fd_response_cb), g_object_get_data (G_OBJECT (data), "eic-gtk"));
  gtk_widget_show (fd);
}

static void
eic_gtk_fd_response_cb (GtkWidget *widget,
                        gint       arg1,
                        gpointer   data)
{
  switch (arg1) {
  case GTK_RESPONSE_ACCEPT: {
    Eic_Gtk_Widgets *eic_gtk;
    gchar           *filename;

    eic_gtk = (Eic_Gtk_Widgets *)data;
    filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (widget));
    gtk_widget_destroy (widget);

    eic_gtk->eic = eic_new (filename);
    eic_fill (eic_gtk->eic);
    eic_gtk_fill (eic_gtk);

    break;
  }
  default: break;
  }
}
