#ifndef __EIC_H__
#define __EIC_H__


#include <glib.h>


#define EIC_TIME_ARGS(t) \
        (int) (((long long)(t))  / (1000000000LL * 60 * 60)), \
        (int) ((((long long)(t)) / (1000000000LL * 60)) % 60), \
        (int) ((((long long)(t)) /  1000000000LL) % 60), \
        (int) (((long long)(t))  %  1000000000LL)

typedef struct Eic_Factory_Info_ Eic_Factory_Info;
typedef struct Eic_Audio_Int_Prop_ Eic_Audio_Int_Prop;
typedef struct Eic_Audio_Float_Prop_ Eic_Audio_Float_Prop;
typedef struct Eic_Audio_Prop_ Eic_Audio_Prop;
typedef struct Eic_Video_Yuv_Prop_ Eic_Video_Yuv_Prop;
typedef struct Eic_Video_Rgb_Prop_ Eic_Video_Rgb_Prop;
typedef struct Eic_Video_Prop_ Eic_Video_Prop;
typedef struct Eic_Prop_ Eic_Prop;

typedef enum
{
  EIC_AUDIO_RAW_INT,
  EIC_AUDIO_RAW_FLOAT
}Eic_Audio_Type;

typedef enum
{
  EIC_VIDEO_RAW_YUV,
  EIC_VIDEO_RAW_RGB
}Eic_Video_Type;

struct Eic_Factory_Info_
{
  const char *name;
  const char *description;
  const char *class;
  char      **authors;
};

struct Eic_Audio_Int_Prop_
{
  int endianess;
  int width;
  int depth;
  int signed_samples; /* boolean */
};

struct Eic_Audio_Float_Prop_
{
  int endianess;
  int width;
  int buffer_frames;
};

struct Eic_Audio_Prop_
{
  Eic_Audio_Type    type;
  const char       *mimetype;
  Eic_Factory_Info *decoder_info;
  int               samplerate;
  int               channels;
  long long         length_time;
  long long         length_samples;
  void             *prop;
};

struct Eic_Video_Yuv_Prop_
{
  char fourcc[4+1];
};

struct Eic_Video_Rgb_Prop_
{
  int bpp;
  int depth;
  int endianess;
  int red_mask;
  int green_mask;
  int blue_mask;
};

struct Eic_Video_Prop_
{
  Eic_Video_Type    type;
  const char       *mimetype;
  Eic_Factory_Info *decoder_info;
  int               width;
  int               height;
  int               framerate_num;
  int               framerate_den;
  int               par_num;
  int               par_den;
  int               dar_num;
  int               dar_den;
  long long         length_time;
  long long         length_frames;
  void             *prop;
};

struct Eic_Prop_
{
  char             *filename;
  int               size;

  Eic_Factory_Info *demuxer_info;

  GList            *video_prop;
  GList            *audio_prop;
};

Eic_Prop *eic_new (const char *filename);
void      eic_free (Eic_Prop *eic);
int       eic_fill (Eic_Prop *eic);

char     *get_size_string (gsize st_size);


#endif /* __EIC_H__ */
